package com.emr.controller;

import com.emr.entity.OffsetLimitPage;
import com.emr.service.ipml.ArchiveFlowInfoService;
import com.emr.util.ExceptionPrintUtil;
import com.emr.util.ExportExcelUtil1;
import com.emr.vo.ArchiveFlowInfoVo;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletResponse;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * @ProjectName:
 * @Description:
 * @Param 传输参数
 * @Return
 * @Author: 曾文和
 * @CreateDate: 2020/5/25 16:18
 * @UpdateUser: 曾文和
 * @UpdateDate: 2020/5/25 16:18
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
@Controller
@RequestMapping("flowInfo/")
public class ArchiveFlowInfoController {
    @Autowired
    private ArchiveFlowInfoService flowInfoService;

    /**
     * 跳转到流程明细页面
     * @return
     */
    @RequestMapping("list")
    public String toList(){
        return "flowInfo/flowInfoList";
    }

    /**
     * 获取流程明细列表
     * @param offset
     * @param limit
     * @param flowInfoVo
     * @param isSearch
     * @return
     * @throws Exception
     */
    @RequestMapping(value = "selectFlowInfoList")
    @ResponseBody
    public OffsetLimitPage selectFlowInfoList(Integer offset, Integer limit, ArchiveFlowInfoVo flowInfoVo,Integer isSearch){
        //判断是否是初始化查询，是初始化查询把开始结束时间置空
        if(isSearch == 0){
            flowInfoVo.setDisStartDate(null);
            flowInfoVo.setDisEndDate(null);
            flowInfoVo.setStartDate(null);
            flowInfoVo.setEndDate(null);
        }
        PageHelper.offsetPage(offset,limit);
        try {
            List<ArchiveFlowInfoVo> list = flowInfoService.selectFlowInfoList(flowInfoVo,null);
            return new OffsetLimitPage((Page) list);
        } catch (Exception e) {
            ExceptionPrintUtil.printException(e);
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 导出流程明细
     * @param response
     * @param flowInfoVo
     * @param sql
     * @param isSearch
     * @throws Exception
     */
    @RequestMapping("exportExcel")
    @ResponseBody
    public void exportExcel(HttpServletResponse response, ArchiveFlowInfoVo flowInfoVo,String sql, Integer isSearch){
        //全部明细
        String tableThNames = "";
        String fieldCns = "";
        //流程明细
        tableThNames = "审核工号,审核姓名,当前操作,审核日期,下个审核节点,患者姓名,住院号,出院科室,出院日期,批注内容,审核用时";
        fieldCns = "userName,checkName,sumbitName,createTimeCn,targetStep,name,inpNo,deptName,dischargeDateTime,remark,useTime";
        //判断是否是初始化查询，是初始化查询把开始结束时间置空
        if(isSearch == 0){
            flowInfoVo.setDisStartDate(null);
            flowInfoVo.setDisEndDate(null);
            flowInfoVo.setStartDate(null);
            flowInfoVo.setEndDate(null);
        }
        try {
            List<ArchiveFlowInfoVo> list = flowInfoService.selectFlowInfoList(flowInfoVo,sql);
            //文件名
            String fileName = "审核流程明细(" + new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date()) + ").xls";
            //ExportExcelUtil
            ExportExcelUtil1 exportExcelUtil = new ExportExcelUtil1();
            //导出excel的操作
            exportExcelUtil.expordExcel(tableThNames,fieldCns,list,fileName,response);
        } catch (Exception e) {
            ExceptionPrintUtil.printException(e);
            e.printStackTrace();
        }
    }
}
