package com.emr.controller;

import com.emr.entity.ArchiveOther;
import com.emr.entity.ArchiveOtherExt;
import com.emr.entity.OffsetLimitPage;
import com.emr.service.ipml.ArchiveOtherExtService;
import com.emr.service.ipml.ArchiveOtherService;
import com.emr.util.ExceptionPrintUtil;
import com.emr.util.Msg;
import com.emr.vo.ArchiveOtherExtVo;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

@Controller
@RequestMapping("archiveExt/")
public class ArchiveOtherExtController {
    @Autowired
    private ArchiveOtherExtService archiveOtherExtService;
    /**
     * 任务详情列表
     * @param offset
     * @param limit
     * @param archiveOtherExtVo
     * @param request
     * @return
     */
    @RequestMapping("getArchiveExtInfo")
    @ResponseBody
    public OffsetLimitPage getArchiveExtInfo(Integer offset, Integer limit, ArchiveOtherExtVo archiveOtherExtVo, HttpServletRequest request){
        if(null != offset && null != limit) {
            PageHelper.offsetPage(offset, limit);
        }

        try {
            List<ArchiveOtherExtVo> list = archiveOtherExtService.getArchiveExtInfo(request,archiveOtherExtVo);
            long offsetLong = offset.longValue();
            long limitLong = limit.longValue();
            OffsetLimitPage page =new OffsetLimitPage(limitLong,offsetLong,list);
            return page;
        } catch (Exception e) {
            ExceptionPrintUtil.printException(e);
            e.printStackTrace();
            return null;
        }
    }
    @RequestMapping("getArchiveExtInfoList")
    @ResponseBody
    public List<ArchiveOtherExtVo> getArchiveExtInfoList(ArchiveOtherExtVo archiveOtherExtVo, HttpServletRequest request){
        try {
            List<ArchiveOtherExtVo> list = archiveOtherExtService.getArchiveExtInfo(request,archiveOtherExtVo);
            return list;
        } catch (Exception e) {
            ExceptionPrintUtil.printException(e);
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 同步更新
     * @param ids 全部选中的ext_id
     * @param notNursingIds 不包含护理ext_id的ext_id
     * @param detailIds detail表的id集合
     * @param jzh 记账号
     * @param masterId master表的id
     * @param sysFlag ext表的权限标识
     * @return
     * @throws Exception
     */
    @RequestMapping("submitUpdate")
    @ResponseBody
    public Msg submitUpdate(String ids,String notNursingIds,String detailIds,String jzh,String masterId,Integer sysFlag,HttpServletRequest request) throws Exception{
        if(StringUtils.isBlank(ids)){
            return Msg.fail("至少选中一个!");
        }
        if(null != sysFlag && sysFlag == 1){
            if(StringUtils.isBlank(jzh)){
                return Msg.fail("记账号不能为空!");
            }
            if(StringUtils.isBlank(masterId)){
                return Msg.fail("masterId不能为空!");
            }
        }
        archiveOtherExtService.updateSubmit(ids,notNursingIds,detailIds,jzh,masterId,sysFlag,request);
        return Msg.success();
    }
    @RequestMapping("submitUpdate1")
    @ResponseBody
    public Msg submitUpdate1(String ids){

        archiveOtherExtService.updateSubmit1(ids);

        return Msg.success();
    }

    /**
     * 根据id查询采集完成记录数
     * @param ids ext_id
     * @return
     * @throws Exception
     */
    @RequestMapping("selectAllByIdsAndSubmitTime")
    @ResponseBody
    public Msg selectSuccessByExtIds(String ids,String jzh) throws Exception{
        if(StringUtils.isNotBlank(ids)){
            String[] split = ids.split(",");
            StringBuilder temp = new StringBuilder();
            for(String str : split){
                if(StringUtils.isNotBlank(str)){
                    if(StringUtils.isNotBlank(temp)){
                        temp.append(",");
                    }
                    temp.append("'").append(str).append("'");
                }
            }
            List<ArchiveOther> list = archiveOtherExtService.selectAllByIdsAndSubmitTime(temp.toString(),jzh);
            return Msg.success().add("list",list);
        }else{
            return Msg.fail("参数ids不能为空");
        }
    }
    /**
     * 根据id查询采集完成记录数
     * @param ids
     * @return
     * @throws Exception
     */
    @RequestMapping("selectAllByIdsAndSubmitTime1")
    @ResponseBody
    public int selectSuccessByExtIds1(String ids) {
        Integer i = archiveOtherExtService.SuccessByExtIds1(ids);
        if (i==0){
            //返回1是采集完毕2还没完成采集
            return 1;
        }else {
            return 0;
        }
    }

    /**
     * 查询未同步更新完成的文件名称集合
     * @param patientId 记账号
     * @return
     * @throws Exception
     */
    @RequestMapping("selectC1WithNotCollectFinishByPatientId/{patientId}")
    @ResponseBody
    public Msg selectC1WithNotCollectFinishByPatientId(@PathVariable("patientId") String patientId) throws Exception{
        String c1s = archiveOtherExtService.selectC1WithNotCollectFinishByPatientId(patientId);
        return Msg.success().add("c1s",c1s);
    }
}
