package com.emr.controller;

import com.alibaba.fastjson.JSON;
import com.emr.entity.ArchiveRemindDeptUser;
import com.emr.entity.EmrOvertimeSet;
import com.emr.service.ipml.EmrOvertimeSetService;
import com.emr.util.ExceptionPrintUtil;
import com.emr.util.Msg;
import com.emr.vo.CommomTree;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import java.util.List;

/**
 * @ProjectName:
 * @Description:
 * @Param 传输参数
 * @Return
 * @Author: 曾文和
 * @CreateDate: 2020/10/27 8:57
 * @UpdateUser: 曾文和
 * @UpdateDate: 2020/10/27 8:57
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
@Controller
@RequestMapping("overtime")
public class OverTimeSetController {
    @Autowired
    private EmrOvertimeSetService overtimeSetService;
    /**
     * 跳转到超期设置页面
     * @param model
     * @return
     */
    @RequestMapping("overtimeSet")
    public String overtimeSet(Model model){
        EmrOvertimeSet overtimeSet = overtimeSetService.selectByPrimaryKey(1);
        model.addAttribute("overtimeSet",overtimeSet);
        return "overtimeSet/overtimeSet";
    }

    /**
     * 修改超期基本信息设置
     * @param overtimeSet
     * @return
     */
    @RequestMapping("updateOverTimeSet")
    @ResponseBody
    public Msg updateOverTimeSet(EmrOvertimeSet overtimeSet) throws Exception{
        overtimeSetService.updateOverTimeSet(overtimeSet);
        return Msg.success();
    }

    /**
     * 查询取消提醒科室树
     * @return
     */
    @RequestMapping("selectRemindDeptTree")
    @ResponseBody
    public List<CommomTree> selectRemindDeptTree(){
        try {
            return overtimeSetService.selectRemindDeptTree();
        } catch (Exception e) {
            ExceptionPrintUtil.printException(e);
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 查询取消提醒科室用户树
     * @return
     */
    @RequestMapping("selectRemindDeptUserTree")
    @ResponseBody
    public List<CommomTree> selectRemindDeptUserTree(){
        try {
            return overtimeSetService.selectRemindDeptUserTree();
        } catch (Exception e) {
            ExceptionPrintUtil.printException(e);
            e.printStackTrace();
            return null;
        }
    }

    /**
     * 保存取消提醒科室集合
     * @return
     */
    @RequestMapping(value="updateRemindCancleSet",method = RequestMethod.POST)
    @ResponseBody
    public Msg updateRemindCancleSet(String deptIds) throws Exception{
        overtimeSetService.updateRemindCancleSet(deptIds);
        return Msg.success();
    }

    /**
     * 保存取消提醒科室单独提醒个别用户集合
     * @return
     */
    @RequestMapping(value="updateRemindDeptUserSet",method = RequestMethod.POST)
    @ResponseBody
    public Msg updateRemindDeptUserSet(String jsonStr) throws Exception{
        //json字符串转换成list集合
        List<ArchiveRemindDeptUser> list = JSON.parseArray(jsonStr, ArchiveRemindDeptUser.class);
        //批量添加
        overtimeSetService.updateRemindDeptUserSet(list);
        return Msg.success();
    }
}
