package com.emr.controller;

import com.emr.dao.ServerMachineMapper;
import com.emr.entity.Archive_Master_Vo;
import com.emr.entity.OffsetLimitPage;
import com.emr.service.ipml.RecordService;
import com.emr.service.ipml.WorkHomeService;
import com.emr.util.ExceptionPrintUtil;
import com.emr.util.ExportExcelUtil;
import com.emr.vo.NotSubmitRecordVo;
import com.emr.vo.RecordSearch;
import com.emr.vo.ServerMachineVo;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.servlet.http.HttpServletResponse;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;


/**
 * @ProjectName:
 * @Description:
 * @Param 传输参数
 * @Return
 * @Author: 曾文和
 * @CreateDate: 2021/7/23 13:23
 * @UpdateUser: 曾文和
 * @UpdateDate: 2021/7/23 13:23
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
@Controller
@RequestMapping("workHome")
public class WorkHomeController {
    @Autowired
    private WorkHomeService workHomeService;
    @Autowired
    private RecordService recordService;
    @Autowired
    private ServerMachineMapper serverMachineMapper;
    /**
     * 工作看板
     * @return
     */
    @RequestMapping("workHomeList")
    public String workHomeList(){
        return "workHome/workHomeList";
    }

    /**
     * 工作看板内容
     * @return
     */
    @RequestMapping("workHomeIframe")
    public String workHomeIframe(){
        return "workHome/workHomeIframe";
    }

    /**
     * 病案数据
     * @return
     */
    @RequestMapping("recordData")
    public String recordData(){
        return "workHome/recordData";
    }

    /**
     * 病案数据第一层表格
     * @return
     */
    @RequestMapping("recordDataOneTable")
    @ResponseBody
    public OffsetLimitPage recordDataOneTable(String startDateTo,String endDateTo) throws Exception{
        return workHomeService.recordDateOneTable(startDateTo,endDateTo);
    }

    /**
     * 病案数据第二层表格
     * @return
     */
    @RequestMapping("recordDataTwoTable")
    @ResponseBody
    public OffsetLimitPage recordDataTwoTable(Integer offset, Integer limit,String startDateTo,String endDateTo) throws Exception{
        return workHomeService.recordDataTwoTable(offset,limit,startDateTo,endDateTo);
    }

    /**
     * 病案数据第三层表格
     * @return
     */
    @RequestMapping("recordDataThreeTable")
    @ResponseBody
    public List<NotSubmitRecordVo> recordDataThreeTable(RecordSearch recordSearch) throws Exception{
        return workHomeService.recordDataThreeTable(recordSearch);
    }

    /**
     * 病案数据待审核详情页面
     * @return
     */
    @RequestMapping("notSubmitRecord")
    public String notSubmitRecord(RecordSearch recordSearch, Model model) throws Exception{
        model.addAttribute("obj",recordSearch);
        return "workHome/notSubmitRecord";
    }

    /**
     * 病案数据未提交报表导出
     * @param response
     */
    @ResponseBody
    @RequestMapping(value = "exportExcelNotSubmit")
    public void exportExcelNotSubmit(HttpServletResponse response,
                                        RecordSearch recordSearch){
        String tableThNames = "主管医生,医生工号,超期份数,出院科室,患者姓名,住院号,"+
                "住院次数,出院日期,超期天数,状态,完整性";
        String fieldCns = "doctorInCharge,doctorInChargeCode,overTimeCounts,deptName,name," +
                "inpNo,visitId,dischargeDateTime,overtimeDays,status,lockInfo";
        //构造excel的数据
        try {
            List<Archive_Master_Vo> list = recordService.selectRecordGroupDeptAndRole(recordSearch);
            //文件名
            String fileName = "病案未提交报表" + new SimpleDateFormat("yyyyMMddHHmmss").format(new Date()) + ".xls";
            //ExportExcelUtil
            ExportExcelUtil exportExcelUtil = new ExportExcelUtil();
            //导出excel的操作
            exportExcelUtil.expordExcel(tableThNames,fieldCns,list,fileName,response);
        }catch (Exception e){
            ExceptionPrintUtil.printException(e);
            e.printStackTrace();
        }
    }

    /**
     * 病案室管理
     * @return
     */
    @RequestMapping("medicalRoomManage")
    public String medicalRoomManage(){
        return "workHome/medicalRoomManage";
    }

    /**
     * 病案室管理数据监控
     * @return
     */
    @RequestMapping("medicalRoomManageData")
    @ResponseBody
    public Map<String,String> medicalRoomManageData(String startDateTo, String endDateTo) throws Exception{
        return workHomeService.medicalRoomManageData(startDateTo,endDateTo);
    }

    /**
     * HIS退回病案统计
     * @return
     */
    @RequestMapping("callBackStatistics")
    public String callBackStatistics(){
        return "workHome/callBackStatistics";
    }

    /**
     * HIS退回病案统计数据
     * @return
     */
    @RequestMapping("callBackStatisticsData")
    @ResponseBody
    public OffsetLimitPage callBackStatisticsData(Integer offset, Integer limit,String startDateTo,String endDateTo) throws Exception{
        return workHomeService.callBackStatisticsData(offset,limit,startDateTo,endDateTo);
    }

    /**
     * HIS退回病案统计数据详情
     * @return
     */
    @RequestMapping("callBackStatisticsDetail")
    @ResponseBody
    public OffsetLimitPage callBackStatisticsDetail(Integer offset, Integer limit,String time,String startDateTo,String endDateTo) throws Exception{
        return workHomeService.callBackStatisticsDetail(offset,limit,time,startDateTo,endDateTo);
    }

    /**
     * 采集器状态
     * @return
     */
    @RequestMapping("collectList")
    public String collectList(){
        return "workHome/collectList";
    }

    /**
     * 采集器状态列表
     * @return
     */
    @RequestMapping("collectListData")
    @ResponseBody
    public OffsetLimitPage collectListData(Integer offset, Integer limit) throws Exception{
        PageHelper.offsetPage(offset, limit);
        List<ServerMachineVo> list = serverMachineMapper.selectAll();
        return new OffsetLimitPage((Page) list);
    }
}
