package com.emr.service;

import com.alibaba.fastjson.JSON;
import com.emr.dao.ArchiveTransferDeptUserMapper;
import com.emr.entity.ArchiveTransferDeptUser;
import com.emr.entity.Emr_Dictionary;
import com.emr.entity.Power_User;
import com.emr.service.ipml.CommomService;
import com.emr.service.ipml.StatisticsService;
import com.emr.vo.CommomTree;
import com.emr.vo.User;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.util.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @ProjectName:
 * @Description:
 * @Param 传输参数
 * @Return
 * @Author: 曾文和
 * @CreateDate: 2020/11/5 17:09
 * @UpdateUser: 曾文和
 * @UpdateDate: 2020/11/5 17:09
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
@Service
@Transactional
public class ArchiveTransferDeptUserService {
    @Autowired
    private Emr_DictionaryService dictionaryService;
    @Autowired
    private ArchiveTransferDeptUserMapper transferDeptUserMapper;
    @Autowired
    private CommomService commomService;
    public List<CommomTree> selectTranferDeptUserTree(String deptName, String masterId) throws Exception {
        //查询科室
        List<Emr_Dictionary> dictionaries = dictionaryService.dicByClo(new Emr_Dictionary());
        List<CommomTree> list = new ArrayList<>();
        if(!CollectionUtils.isEmpty(dictionaries)){
            //查询用户信息
            List<User> userList = commomService.getUserList();
            if(!CollectionUtils.isEmpty(userList)){
                //查询有选中的用户
                List<ArchiveTransferDeptUser> transferDeptUsers = transferDeptUserMapper.selectAllByMasterId(masterId);
                //定义角色id集合
                String roleListJson = "[" +
                        "{'roleId':1,roleName:'医生'}," +
                        "{'roleId':5,roleName:'医生质控员'}," +
                        "{'roleId':6,roleName:'科主任'}," +
                        "{'roleId':2,roleName:'护士'}," +
                        "{'roleId':8,roleName:'护士质控员'}," +
                        "{'roleId':7,roleName:'护士长'}," +
                        "{'roleId':3,roleName:'病案室人员'}" +
                        "]";
                List<User> roleList = JSON.parseArray(roleListJson, User.class);
                //设定用户id从20000开始
                int id = 1;
                int twoParentId = 0;
                int threeParentId = 0;
                //组织用户
                for(Emr_Dictionary dictionary : dictionaries) {
                    //获取科室编码
                    String code = dictionary.getCode();
                    if(StringUtils.isNotBlank(code) && !dictionary.getName().equals(deptName)) {
                        //第一层，科室层
                        CommomTree tree1 = new CommomTree();
                        tree1.setId(id);
                        twoParentId = id;
                        id++;
                        tree1.setCode(dictionary.getCode());
                        tree1.setName(dictionary.getName());
                        tree1.setParentId(0);
                        //过滤字典类别科室 科室code=null
                        //定义存在用户的科室
                        boolean deptFlag = false;
                        boolean deptCheckedFlag = false;
                        for(User role : roleList){
                            //第二层，审核角色层
                            CommomTree tree2 = new CommomTree();
                            tree2.setId(id);
                            threeParentId = id;
                            id++;
                            tree2.setParentId(twoParentId);
                            tree2.setName(role.getRoleName());
                            //定义存在用户的角色
                            boolean roleFlag = false;
                            boolean roleCheckedFlag = false;
                            for (User user : userList) {
                                //第三层，用户层
                                //获取用户所属科室
                                String deptCode = user.getDeptCode();
                                if (StringUtils.isNotBlank(deptCode) && deptCode.equals(code) && role.getRoleId().equals(user.getRoleId())) {
                                    CommomTree tree3 = new CommomTree();
                                    tree3.setId(id);
                                    id++;
                                    tree3.setParentId(threeParentId);
                                    String name = user.getName();
                                    if(StringUtils.isNotBlank(name)){
                                        tree3.setName(user.getUserName() + "-" + name);
                                    }else{
                                        tree3.setName(user.getUserName());
                                    }
                                    //设置工号
                                    tree3.setRealName(user.getUserName());
                                    //设置科室编号
                                    tree3.setCode(deptCode);
                                    //设置角色id
                                    tree3.setRealParentId(user.getRoleId());
                                    //判断是否用户选中
                                    if(!CollectionUtils.isEmpty(transferDeptUsers)){
                                        for(ArchiveTransferDeptUser userTemp : transferDeptUsers){
                                            if(userTemp.getUserName().equals(user.getUserName())){
                                                tree3.setChecked("true");
                                                deptCheckedFlag = true;
                                                roleCheckedFlag = true;
                                                break;
                                            }
                                        }
                                    }
                                    list.add(tree3);
                                    deptFlag = true;
                                    roleFlag = true;
                                }
                            }
                            //角色不存在用户则移除。不添加
                            if(roleFlag){
                                if(roleCheckedFlag) {
                                    tree2.setChecked("true");
                                }
                                list.add(tree2);
                            }
                        }
                        //科室不存在用户则移除。不添加
                        if(deptFlag){
                            if(deptCheckedFlag) {
                                tree1.setChecked("true");
                            }
                            list.add(tree1);
                        }
                    }
                }
            }
        }
        return list;
    }

    /**
     * 保存跨科查看用户信息
     * @param list
     */
    public void updateTransferDeptUserSet(List<ArchiveTransferDeptUser> list, HttpServletRequest request) {
        if(!CollectionUtils.isEmpty(list)) {
            String masterId = list.get(0).getMasterId();
            //先清空表
            transferDeptUserMapper.deleteTransferDeptUserByMasterId(masterId);
            //填充数据
            //获取用户
            Power_User user = (Power_User)request.getSession().getAttribute("CURRENT_USER");
            //获取当前时间
            Date date = new Date();
            for(ArchiveTransferDeptUser obj : list){
                obj.setCreateTime(date);
                obj.setCreateUser(user.getUserName());
            }
            //批量插入
            commomService.insertBatch("archive_transfer_dept_user", list, transferDeptUserMapper);
        }
    }
}
