package com.emr.service.ipml;

import com.emr.dao.ArchiveCallbackInfoMapper;
import com.emr.dao.ArchiveFlowRoleMapper;
import com.emr.entity.ArchiveCallbackInfo;
import com.emr.entity.ArchiveFlowRole;
import com.emr.entity.Archive_Master;
import com.emr.entity.OffsetLimitPage;
import com.emr.service.Archive_MasterService;
import com.emr.vo.ArchiveCallbackInfoVo;
import com.emr.vo.FinalAndFirstStatistics;
import com.emr.vo.User;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.util.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.Date;
import java.util.List;

/**
 * @ProjectName:
 * @Description:
 * @Param 传输参数
 * @Return
 * @Author: 曾文和
 * @CreateDate: 2020/5/12 16:12
 * @UpdateUser: 曾文和
 * @UpdateDate: 2020/5/12 16:12
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
@Service
@Transactional
public class ArchiveCallbackInfoService {
    @Autowired
    private ArchiveCallbackInfoMapper callbackInfoMapper;
    @Autowired
    private Archive_MasterService archiveMasterService;
    @Autowired
    private ArchiveFlowInfoService archiveFlowInfoService;
    @Autowired
    private CommomService commomService;
    @Autowired
    private ArchiveFlowRoleMapper flowRoleMapper;
    /**
     * 添加召回记录
     *
     * @param callbackInfo
     * @throws Exception
     */
    public void insertCallbackInfoService(ArchiveCallbackInfo callbackInfo) throws Exception {
        String username = (String) SecurityUtils.getSubject().getSession().getAttribute("userSession");
        callbackInfo.setCallbackUserName(username);
        callbackInfo.setCallbackTime(new Date());
        //获取ip
        String addressIp = getAddressIp();
        callbackInfo.setIp(addressIp);
        callbackInfoMapper.insert(callbackInfo);
        //将病案置为召回，修改状态码
        Archive_Master master = new Archive_Master();
        master.setId(callbackInfo.getMasterId());
        master.setArchivestate(EnumVerify.DocState.CallBacked.getCode() + "");
        archiveMasterService.updateByClo(master);
        //添加进流程
        String masterId = callbackInfo.getMasterId();
        String currentCode = EnumVerify.DocState.CallBacked.getCode() + "";
        String remark = callbackInfo.getCallbackReason();
        Integer targetCode = EnumVerify.DocState.CallBacked.getCode();
        //获取当前节点和下个节点对象集合
        List<ArchiveFlowRole> archiveFlowRoles = flowRoleMapper.selectAll();
        List<ArchiveFlowRole> currentRoleAndNextRole = archiveFlowInfoService.getCurrentRoleAndNextRole(currentCode,targetCode,archiveFlowRoles);
        archiveFlowInfoService.insertFlowInfo(masterId,remark,currentRoleAndNextRole.get(0),currentRoleAndNextRole.get(1),new Date());
        //TODO 删除病案资料
        callbackInfoMapper.deleteRecordByRecall(masterId);
        callbackInfoMapper.deleteRecordByInp(masterId);
    }

    private String getAddressIp() throws Exception{
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        // 获取访问真实IP
        String ipAddress = request.getHeader("x-forwarded-for");
        if(ipAddress == null || ipAddress.length() == 0 || "unknown".equalsIgnoreCase(ipAddress)) {
            ipAddress = request.getHeader("Proxy-Client-IP");
        }
        if(ipAddress == null || ipAddress.length() == 0 || "unknown".equalsIgnoreCase(ipAddress)) {
            ipAddress = request.getHeader("WL-Proxy-Client-IP");
        }
        if(ipAddress == null || ipAddress.length() == 0 || "unknown".equalsIgnoreCase(ipAddress)) {
            ipAddress = request.getRemoteAddr();
            if(ipAddress.equals("127.0.0.1") || ipAddress.equals("0:0:0:0:0:0:0:1")){
                //根据网卡取本机配置的IP
                InetAddress inet = InetAddress.getLocalHost();
                ipAddress= inet.getHostAddress();
            }
        }
        //对于通过多个代理的情况，第一个IP为客户端真实IP,多个IP按照','分割
        if(ipAddress!=null && ipAddress.length()>15){ //"***.***.***.***".length() = 15
            if(ipAddress.indexOf(",")>0){
                ipAddress = ipAddress.substring(0,ipAddress.indexOf(","));
            }
        }
        return ipAddress;
    }
    /**
     * 查询列表
     *
     * @param callbackInfoVo
     * @param offset
     * @param limit
     * @return
     */
    public OffsetLimitPage selectAll(ArchiveCallbackInfoVo callbackInfoVo, Integer offset, Integer limit) throws Exception{
        PageHelper.offsetPage(offset, limit);
        List<ArchiveCallbackInfoVo> list = selectCallBackInfo(callbackInfoVo);
        return new OffsetLimitPage((Page) list);
    }

    public List<ArchiveCallbackInfoVo> selectCallBackInfo(ArchiveCallbackInfoVo callbackInfoVo) throws Exception{
        List<ArchiveCallbackInfoVo> list = callbackInfoMapper.selectAll(callbackInfoVo);
        if (!CollectionUtils.isEmpty(list)) {
            //转换状态
            //计算属于哪种审核角色
            List<User> userList = commomService.getUserList();
            for (ArchiveCallbackInfoVo callbackInfoVo1 : list) {
                String archivestate = callbackInfoVo1.getArchivestate();
                if (StringUtils.isNotBlank(archivestate)) {
                    //转换中文状态
                    Integer status = Integer.valueOf(archivestate);
                    archivestate = EnumVerify.DocState.GetStepName(status, EnumVerify.Checker.Generate.getCode());
                    callbackInfoVo1.setArchivestate(archivestate);
                }
                if (!CollectionUtils.isEmpty(userList)) {
                    //转换姓名
                    String userName = callbackInfoVo1.getCallbackUserName();
                    for (User user : userList) {
                        if (StringUtils.isNotBlank(userName) && userName.equals(user.getUserName())) {
                            callbackInfoVo1.setCallbackUserName(user.getName());
                            break;
                        }
                    }
                }
            }
        }
        return list;
    }
}
