package com.emr.service.ipml;

import com.emr.dao.ArchiveFlowInfoMapper;
import com.emr.dao.ArchiveFlowRoleMapper;
import com.emr.dao.Archive_MasterMapper;
import com.emr.dao.EmrHolidaySetMapper;
import com.emr.entity.*;
import com.emr.service.Emr_Fault_TypeService;
import com.emr.vo.ArchiveFlowInfoVo;
import com.emr.vo.User;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.util.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedList;
import java.util.List;

/**
 * @ProjectName:
 * @Description:
 * @Param 传输参数
 * @Return
 * @Author: 曾文和
 * @CreateDate: 2020/5/12 8:49
 * @UpdateUser: 曾文和
 * @UpdateDate: 2020/5/12 8:49
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
@Service
@Transactional
public class ArchiveFlowInfoService {
    @Autowired
    private ArchiveFlowInfoMapper flowInfoMapper;
    @Autowired
    private Archive_MasterMapper archiveMasterMapper;
    @Autowired
    private CommomService commomService;
    @Autowired
    private Emr_Fault_TypeService faultTypeService;
    @Autowired
    private EmrHolidaySetMapper holidaySetMapper;
    /**
     * 添加进流转记录
     * @param masterId
     * @param remark 审核内容
     * @param currentRole 当前节点对象
     * @param nextRole 手动指定下个节点对象，未指定查询流转下个节点，有指定使用指定
     * @param sumbitDate 提交时间
     * @throws Exception
     */
    public ArchiveFlowInfo insertFlowInfo(String masterId,String remark,ArchiveFlowRole currentRole,ArchiveFlowRole nextRole,Date sumbitDate)throws Exception{
        //添加流程
        ArchiveFlowInfo flowInfo = new ArchiveFlowInfo();
        //masterId;
        flowInfo.setMasterId(masterId);

        //useSeconds:距离上次提交用时s数
        long millis = getMillis(masterId, currentRole, nextRole);
        if(millis != 0){
            flowInfo.setUseSeconds(millis / 1000);
        }
        //如果下节点role不为空取下节点operRole
        if(null != nextRole.getRole()){
            flowInfo.setOperRole(nextRole.getRole());
        }else{
            flowInfo.setOperRole(currentRole.getRole());
        }
        //如果下节点的sumBitName不为空取下节点的sumbitName
        if(StringUtils.isNotBlank(nextRole.getSumbitName())){
            flowInfo.setSumbitName(nextRole.getSumbitName());
        }else{
            flowInfo.setSumbitName(currentRole.getSumbitName());
        }
        //当前stepIndex
        flowInfo.setStepIndex(currentRole.getStepIndex());
        //当前节点id
        flowInfo.setStartStepId(currentRole.getId());
        //startStep:当前步骤name
        flowInfo.setStartStep(currentRole.getName());
        //targetStep
        flowInfo.setTargetStep(nextRole.getName());
        //下个节点的id
        flowInfo.setTargetStepId(nextRole.getId());
        //remark:提交内容
        flowInfo.setRemark(remark);
        //userName:提交人用户名
        // 从session获取用户名
        String username = (String) SecurityUtils.getSubject().getSession().getAttribute("userSession");
        flowInfo.setUserName(username);
        //submit:提交时间
        flowInfo.setSumbitTime(sumbitDate);
        //createtime:创建时间
        flowInfo.setCreatetime(new Date());
        flowInfoMapper.insertSelective(flowInfo);
        return flowInfo;
    }

    /**
     * 获取当前节点(list.get(0))和下个节点对象信息(list.get(1))
     * @param currentStatus
     * @param appointNextCode
     * @return
     */
    public List<ArchiveFlowRole> getCurrentRoleAndNextRole(String currentStatus,Integer appointNextCode,List<ArchiveFlowRole> archiveFlowRoles){
        List<ArchiveFlowRole> currentRoleAndNextRoleList = new LinkedList<>();
        //查询审核角色集合，根据有效进行计算状态code
        //stepIndex:当前步骤
        ArchiveFlowRole currentRole = getCurrentRoleByCurrentStatus(currentStatus, archiveFlowRoles);
        currentRoleAndNextRoleList.add(currentRole);
        //targetStep:下一步骤节点信息
        ArchiveFlowRole nextRole = getNextRoleByCurrentStatus(appointNextCode, archiveFlowRoles, currentRole);
        currentRoleAndNextRoleList.add(nextRole);
        return currentRoleAndNextRoleList;
    }

    /***
     * 计算节点间间隔毫秒数
     * @param masterId
     * @param currentRole
     * @param nextRole
     * @return
     * @throws ParseException
     */
    private long getMillis(String masterId, ArchiveFlowRole currentRole, ArchiveFlowRole nextRole) throws ParseException {
        long millis = 0;
        Date preDate = new Date();
        //查询上次节点的时间
        //查询上次节点的时间方法参数当前节点的名称，当下个节点为病案室终审不带当前审核名称查询该病案最后审核时间
        List<ArchiveFlowInfo> archiveFlowInfos = new ArrayList<>();
        String currentName = "";
        String name = nextRole.getName();
        if (StringUtils.isNotBlank(name) && !name.equals(EnumVerify.DocState.MedicalRoomCheck.getName())) {
            currentName = currentRole.getName();
        }
        archiveFlowInfos = flowInfoMapper.selectPreInfo(masterId, currentName);
        if (null != archiveFlowInfos && !archiveFlowInfos.isEmpty()) {
            //存在取最后一次提交时间
            preDate = archiveFlowInfos.get(0).getCreatetime();
        } else {
            //取出院时间
            Archive_Master archiveMaster1 = archiveMasterMapper.selectByPrimaryKey(masterId);
            SimpleDateFormat fmt = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            String dischargeDateTime = archiveMaster1.getDischargeDateTime();
            if (StringUtils.isNotBlank(dischargeDateTime)) {
                preDate = fmt.parse(dischargeDateTime);
            }
        }
        if (null != preDate) {
            millis = System.currentTimeMillis() - preDate.getTime();
            //查询是否这个时间段是否有假日
            int holidayDays = holidaySetMapper.selectHolidayByDate(preDate);
            if(holidayDays != 0){
                int holidayMilTime = holidayDays * 24 * 3600 * 1000;
                millis -= holidayMilTime;
            }
        }
        return millis;
    }

    /**
     * 根据让钱节点信息获取获取下个节点信息
     * @param appointNextCode
     * @param archiveFlowRoles
     * @param currentRole
     * @return
     */
    private ArchiveFlowRole getNextRoleByCurrentStatus(Integer appointNextCode, List<ArchiveFlowRole> archiveFlowRoles, ArchiveFlowRole currentRole) {
        ArchiveFlowRole nextRole = new ArchiveFlowRole();
        for (ArchiveFlowRole archiveFlowRole : archiveFlowRoles) {
            //判断相同审核角色，比当前步骤还大且有效的角色
            if (null != archiveFlowRole.getRole() && archiveFlowRole.getEffective() == 1) {
                if(null == appointNextCode){
                    if(null != currentRole.getRole() && null != archiveFlowRole.getStepIndex()){
                        if (currentRole.getRole().equals(archiveFlowRole.getRole()) && archiveFlowRole.getStepIndex() > currentRole.getStepIndex()) {
                            nextRole = archiveFlowRole;
                            nextRole.setRole(null);
                            nextRole.setSumbitName(null);
                            break;
                        }
                    }
                }else{
                    //指定的下个节点code用于退回
                    if (archiveFlowRole.getCode().equals(appointNextCode+"")) {
                        //医生退回下个节点为医生
                        if(appointNextCode.equals(EnumVerify.DocState.DortorBack.getCode())){
                            //sumbitName
                            nextRole.setSumbitName(EnumVerify.DocState.DortorBack.getName());
                            nextRole.setName(EnumVerify.DocState.DortorSubmit.getName());
                            nextRole.setId(archiveFlowRole.getId());
                            nextRole.setRole(Short.valueOf("1"));
                            break;
                            //护士退回下个节点为护士
                        }else if(appointNextCode.equals(EnumVerify.DocState.NurseBack.getCode())){
                            nextRole.setSumbitName(EnumVerify.DocState.NurseBack.getName());
                            nextRole.setName(EnumVerify.DocState.NurseSubmit.getName());
                            nextRole.setId(archiveFlowRole.getId());
                            nextRole.setRole(Short.valueOf("2"));
                            break;
                            //下个节点是病案室审核，属于召回
                        }else if(appointNextCode.equals(EnumVerify.DocState.CallBacked.getCode())){
                            nextRole.setSumbitName(archiveFlowRole.getSumbitName());
                            nextRole.setName(EnumVerify.DocState.MedicalRoomCheck.getName());
                            //查询病案室终审id
                            for (ArchiveFlowRole archiveFlowRole1 : archiveFlowRoles) {
                                if (archiveFlowRole1.getCode().equals(EnumVerify.DocState.MedicalRoomCheck.getCode()+"")) {
                                    nextRole.setId(archiveFlowRole1.getId());
                                    break;
                                }
                            }
                            //病案室审核
                        }else if(appointNextCode.equals(EnumVerify.DocState.MedicalRoomCheck.getCode())){
                            nextRole.setName("已归档");
                            nextRole.setId(archiveFlowRole.getId());
                        }
                    }
                }
            }
        }
        //下个节点为空，填写护士长
        if(StringUtils.isBlank(nextRole.getName())){
            for (ArchiveFlowRole archiveFlowRole : archiveFlowRoles) {
                if(archiveFlowRole.getCode().equals(EnumVerify.DocState.HeadNurseCheck.getCode()+"")){
                    nextRole = archiveFlowRole;
                    nextRole.setRole(null);
                    nextRole.setSumbitName(null);
                    break;
                }
            }
        }
        return nextRole;
    }

    /**
     * 根据当前状态码获取当前角色节点信息
     * @param currentStatus
     * @param archiveFlowRoles
     * @return
     */
    private ArchiveFlowRole getCurrentRoleByCurrentStatus(String currentStatus, List<ArchiveFlowRole> archiveFlowRoles) {
        ArchiveFlowRole currentRole = new ArchiveFlowRole();
        for (ArchiveFlowRole archiveFlowRole : archiveFlowRoles) {
            if (archiveFlowRole.getCode().equals(currentStatus)) {
                currentRole = archiveFlowRole;
                break;
            }
        }
        return currentRole;
    }


    /**
     * 根据masterId查看流转流程信息
     * @param id
     * @return
     */
    public List<ArchiveFlowInfoVo> selectFlowInfoByMasterId(String id,String userNameTemp) throws Exception{
        List<ArchiveFlowInfoVo> list = flowInfoMapper.selectFlowInfoByMasterId(id);
        if(!CollectionUtils.isEmpty(list)) {
            //获取缺陷类别列表
            Emr_Fault_Type obj = new Emr_Fault_Type();
            obj.setEffective(1);
            List<Emr_Fault_Type> typeList = faultTypeService.selectByCol(obj);
            List<User> userList = commomService.getUserList();
            //替换姓名
            if (!CollectionUtils.isEmpty(userList)) {
                for (ArchiveFlowInfoVo infoVo : list) {
                    //转换姓名
                    String userName = infoVo.getUserName();
                    for (User user : userList) {
                        if (StringUtils.isNotBlank(userName) && userName.equals(user.getUserName())) {
                            infoVo.setUserName(user.getName());
                            break;
                        }
                    }
                    //转换缺陷类别
                    String faultAssortId = infoVo.getFaultAssortName();
                    if (StringUtils.isNotBlank(faultAssortId)) {
                        String[] faultAssortIds = faultAssortId.split(",");
                        StringBuilder falutAssortName = new StringBuilder();
                        for (String assortId : faultAssortIds) {
                            for (Emr_Fault_Type faultType : typeList) {
                                if (assortId.equals(faultType.getId() + "")) {
                                    falutAssortName.append(faultType.getTypeName()).append(",");
                                    break;
                                }
                            }
                        }
                        if (falutAssortName.toString().endsWith(",")) {
                            falutAssortName = new StringBuilder(falutAssortName.substring(0, falutAssortName.length() - 1));
                            infoVo.setFaultAssortName(falutAssortName.toString());
                        }
                    }
                }
            }
        }
        return list;
    }

    /**
     * 获取流程明细列表
     * @param flowInfoVo
     * @return
     */
    public List<ArchiveFlowInfoVo> selectFlowInfoList(ArchiveFlowInfoVo flowInfoVo,String sql) throws Exception{
        //模糊搜索审核姓名转换工号
        String checkName = flowInfoVo.getUserName();
        if(StringUtils.isNoneBlank(checkName)){
            //转换姓名
            List<User> userList = commomService.getUserList();
            StringBuilder checkNames = new StringBuilder();
            for(User user:userList){
                if(StringUtils.isNoneBlank(user.getName()) && user.getName().contains(checkName)){
                    checkNames.append("'").append(user.getUserName()).append("',");
                }
            }
            if(StringUtils.isNotBlank(checkNames)){
                checkNames = new StringBuilder(checkNames.substring(0,checkNames.length()-1));
                flowInfoVo.setUserName(checkNames.toString());
            }else{
                flowInfoVo.setUserName("'" + checkName + "'");
            }
        }
        List<ArchiveFlowInfoVo> list = flowInfoMapper.selectFlowInfoList(flowInfoVo,sql);
        //替换姓名
        if(!CollectionUtils.isEmpty(list)) {
            List<User> userList = commomService.getUserList();
            if (!CollectionUtils.isEmpty(userList)) {
                for (ArchiveFlowInfoVo infoVo : list) {
                    //转换姓名
                    String userName = infoVo.getUserName();
                    for (User user : userList) {
                        if (StringUtils.isNotBlank(userName) && userName.equals(user.getUserName())) {
                            infoVo.setCheckName(user.getName());
                            break;
                        }
                    }
                }
            }
        }
        return list;
    }
}
