package com.emr.service.ipml;

import com.emr.dao.ArchiveOtherMapper;
import com.emr.entity.ArchiveOther;
import com.emr.entity.ArchiveOtherExt;
import com.emr.util.IDHelper;
import com.emr.vo.ArchiveOtherExtVo;
import org.apache.commons.lang.StringUtils;
import org.apache.shiro.util.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * @ProjectName:
 * @Description:
 * @Param 传输参数
 * @Return
 * @Author: 曾文和
 * @CreateDate: 2020/7/8 10:33
 * @UpdateUser: 曾文和
 * @UpdateDate: 2020/7/8 10:33
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
@Service
@Transactional
public class ArchiveOtherService {
    @Autowired
    private ArchiveOtherMapper archiveOtherMapper;

    /**
     * 存在将C1置为'' 不存在新增一条记录
     * @param patientId 记账号
     */
    public void updateArchiveOther(String patientId,String masterId) throws Exception{
        //查询记账号是否存在
        List<ArchiveOther> archiveOthers = archiveOtherMapper.selectExistByPatientId(patientId);
        if(!CollectionUtils.isEmpty(archiveOthers)){
            //存在则将C1值置为空字符串
            ArchiveOther archiveOther = archiveOthers.get(0);
            archiveOther.setC1("");
            archiveOtherMapper.updateByPrimaryKeySelective(archiveOther);
        }else{
            //不存在则新增
            ArchiveOtherExtVo extObj = new ArchiveOtherExtVo();
            extObj.setJzh(patientId);
            extObj.setMid(masterId);
            //填充数据
            ArchiveOther archiveOther = getArchiveOther(extObj);
            archiveOtherMapper.insert(archiveOther);
        }
    }

    /**
     * 填充ArchiveOtherExt对象数据
     * @param extObj
     * @return
     * @throws Exception
     */
    public ArchiveOther getArchiveOther(ArchiveOtherExtVo extObj) throws Exception {
        ArchiveOther archiveOther = new ArchiveOther();
        long id = IDHelper.NewID();
        archiveOther.setId(id);
        archiveOther.setCreatedtime(new Date());
        archiveOther.setDetailid("0");
        archiveOther.setDtime(new Date());
        archiveOther.setC1("");
        archiveOther.setC2("");
        archiveOther.setC3("");
        archiveOther.setC4("");
        archiveOther.setC5("");
        archiveOther.setN1(BigDecimal.valueOf(0));
        archiveOther.setN2(BigDecimal.valueOf(0));
        archiveOther.setN3(BigDecimal.valueOf(0));
        String date = "1801-02-03 00:00:00";
        SimpleDateFormat fmt = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        archiveOther.setT1(fmt.parse(date));
        archiveOther.setT2(fmt.parse(date));
        archiveOther.setT3(fmt.parse(date));
        archiveOther.setIsfulltext(false);
        //不同部分
        archiveOther.setPid(extObj.getJzh());
        archiveOther.setMasterid(extObj.getMid());
        Integer sysflag = extObj.getSysflag();
        //定义不同的sysFlag
        Integer sysFlagTemp = null;
        String otherInfo = "";
        if(sysflag == null){
            //护理系统
            sysFlagTemp = -2;
            otherInfo = "护理";
        }else{
            //设置C2的值为other_ext表的id
            archiveOther.setC2(extObj.getId().toString());
            //2 His，3 Pasc，4 心电，5 手麻，6 Lis，7 首页，8 医嘱
            //-2 护理，-3 Pasc，-4 心电，-5 手麻，-6 Lis，-7 首页，-8 医嘱，-9 His
            switch (sysflag){
                case 2:
                    sysFlagTemp = -9;
                    otherInfo = "His";
                    break;
                case 3:
                    sysFlagTemp = -3;
                    otherInfo = "Pasc";
                    break;
                case 4:
                    sysFlagTemp = -4;
                    otherInfo = "心电";
                    break;
                case 5:
                    sysFlagTemp = -5;
                    otherInfo = "手麻";
                    break;
                case 6:
                    sysFlagTemp = -6;
                    otherInfo = "Lis";
                    break;
                case 7:
                    sysFlagTemp = -7;
                    otherInfo = "首页";
                    break;
                case 8:
                    sysFlagTemp = -8;
                    otherInfo = "医嘱";
                    break;
                default:
                    break;
            }
        }
        archiveOther.setOtherinfo(otherInfo+"系统按需采集");
        archiveOther.setSysflag(sysFlagTemp);
        return archiveOther;
    }

    /**
     * @param patientId 记账号
     * @return
     */
    public int selectC1ByPatientId(String patientId) {
        List<ArchiveOther> archiveOthers = archiveOtherMapper.selectExistByPatientId(patientId);
        if(!CollectionUtils.isEmpty(archiveOthers)){
            //获取对象
            ArchiveOther archiveOther = archiveOthers.get(0);
            //获取C1值
            String c1 = archiveOther.getC1();
            //定义c1需求值
            String c1Temp = "done";
            if(StringUtils.isNotBlank(c1) && c1Temp.equals(c1.toLowerCase())){
                return 1;
            }
        }
        return 0;
    }

    /**
     * 根据c2字段的查询集合
     * @param c2s
     * @param c1
     * @return
     */
    public List<ArchiveOther> selectAllByC2(String c2s,String c1){
        return archiveOtherMapper.selectAllByC2(c2s,c1);
    }

    /**
     * 根据护理记录
     * @param c2s
     */
    public void updateBatchC1ByC2s(String c2s){
        archiveOtherMapper.updateBatchC1ByC2s(c2s);
    }
}
