/**
 * Copyright (C), 2015-2019
 * Author:   HJL
 * Date:     2019/7/19 15:37
 * Description:病历文件信息管理
 */
package com.emr.service.ipml;

import com.emr.dao.Archive_DetailMapper;
import com.emr.dao.Emr_Commom_SetMapper;
import com.emr.entity.*;
import com.emr.service.Archive_DetailService;
import com.emr.util.Jpg2PdfUtil;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.util.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import java.util.ArrayList;
import java.util.List;

@Service
public class Archive_DetailServiceImpl implements Archive_DetailService {
    @Autowired
    private Archive_DetailMapper archiveDetailMapper;
    @Autowired
    private Emr_Commom_SetMapper commomSetMapper;
    @Autowired
    private ZdAssortService zdAssortService;
    @Override
    public List<Archive_Detail_Vo> selectByClo(Archive_Detail_Vo record, HttpServletRequest request) {
        List<Archive_Detail_Vo> list = archiveDetailMapper.selectByClo(record);
        //定义权限分类集合
        List<Archive_Detail_Vo> returnList = selectPowerAssort(request, list);
        //页码为0的计算页码
        if(!CollectionUtils.isEmpty(returnList)) {
            for (Archive_Detail_Vo vo : returnList) {
                //页码为0的读取pdf文件重置页码
                setPdfPages(vo);
            }
        }
        return returnList;
    }

    /**
     * 转换权限分类
     * @param request
     * @param list
     * @return
     */
    private List<Archive_Detail_Vo> selectPowerAssort(HttpServletRequest request, List<Archive_Detail_Vo> list) {
        List<Archive_Detail_Vo> returnList = new ArrayList<>();
        if(!CollectionUtils.isEmpty(list)){
            //查询登陆者
            List<Zd_Assort> zdAssorts = zdAssortService.selectAssortListByUserId(request);
            if(!CollectionUtils.isEmpty(zdAssorts)){
                //有权限组织权限分类
                for(Archive_Detail_Vo obj : list){
                    String assortId = obj.getAssortId();
                    //定义是否有权限
                    boolean flag = false;
                    for(Zd_Assort temp : zdAssorts){
                        if(temp.getAssortId().equals(assortId)){
                            flag = true;
                            break;
                        }
                    }
                    if(flag){
                        returnList.add(obj);
                    }
                }
            }else{
                returnList = list;
            }
        }
        return returnList;
    }

    /**
     * 页码为0的读取pdf文件重置页码
     * @param vo
     */
    private void setPdfPages(Archive_Detail_Vo vo) {
        //获取页码
        String pageCounts = vo.getPageCounts();
        //获取路径
        String pdfPaths = vo.getPdfPaths();
        //遍历是否存在页码为0的
        String[] pageCountArr = pageCounts.split(",");
        String[] pdfPathArr = pdfPaths.split(",");
        //定义临时页码集合
        StringBuilder newPageCount = new StringBuilder();
        for (int i = 0; i < pageCountArr.length; i++) {
            String pageCountStr = pageCountArr[i];
            /*if("0".equals(pageCountStr)){
                String pathPath = pdfPathArr[i];
                File file = new File(pathPath);
                try{
                    PdfReader pdfReader = new PdfReader(new FileInputStream(file));
                    pageCountStr = pdfReader.getNumberOfPages() + "";
                }catch (Exception e){
                    e.printStackTrace();
                }
            }*/
            if(StringUtils.isNotBlank(newPageCount)){
                newPageCount.append(",");
            }
            newPageCount.append(pageCountStr);
        }
        //重置页码
        vo.setPageCounts(newPageCount.toString());
    }

    @Override
    public OffsetLimitPage detailByClo(Archive_Detail_Vo record, Integer offset, Integer limit,HttpServletRequest request) {
        Power_User currentUser = (Power_User)request.getSession().getAttribute("CURRENT_USER");
        Integer roleId = currentUser.getRoleId();
        //不是管理员需过滤分类权限
        if(roleId != 0 && roleId != -100){
            record.setUserId(currentUser.getUserId());
            record.setRoleId(currentUser.getRoleId());
        }
        PageHelper.offsetPage(offset, limit);
        List<Archive_Detail_Vo> list = archiveDetailMapper.detailByClo(record);
        OffsetLimitPage offsetLimitPage = new OffsetLimitPage((Page) list);
        if(!CollectionUtils.isEmpty(list)) {
            //TODO 定义新的集合。病程记录分类中的首次病程记录排前头
            String strTemp = "首次病程";
            String assortId = "AFB9FBE656D7492C80AEDE6E685A851A";
            //定义护理记录里的首次护理记录排前头
            String nurseTemp = "首次护理";
            String nurseAssortId = "C70E8C427A3648B79BE80798C08F4D12";
            List<Archive_Detail_Vo> returnList = new Page<>();
            //定义接收首次病程记录的对象
            Archive_Detail_Vo detailVoTemp = new Archive_Detail_Vo();
            Archive_Detail_Vo nurseDetailVoTemp = new Archive_Detail_Vo();
            for (Archive_Detail_Vo detailVo1 : list) {
                if (detailVo1.getAssortId().equals(assortId) && detailVo1.getTitle().contains(strTemp)) {
                    detailVoTemp = detailVo1;
                }
                if (detailVo1.getAssortId().equals(nurseAssortId) && detailVo1.getTitle().contains(nurseTemp)) {
                    nurseDetailVoTemp = detailVo1;
                }
                //页码为0的读取pdf文件重置页码
                setPdfPages(detailVo1);
            }
            if (!CollectionUtils.isEmpty(list)) {
                //重新排序
                //定义是否已添加首次病程
                boolean flag = false;
                boolean nurseFlag = false;
                for (Archive_Detail_Vo detailVo : list) {
                    if (StringUtils.isNotBlank(detailVo.getAssortId()) && detailVo.getAssortId().equals(assortId) && !flag && StringUtils.isNotBlank(detailVoTemp.getId())) {
                        returnList.add(detailVoTemp);
                        flag = true;
                    }
                    if (StringUtils.isNotBlank(detailVo.getAssortId()) && detailVo.getAssortId().equals(nurseAssortId) && !nurseFlag && StringUtils.isNotBlank(nurseDetailVoTemp.getId())) {
                        returnList.add(nurseDetailVoTemp);
                        nurseFlag = true;
                    }
                    if(StringUtils.isNotBlank(detailVo.getTitle()) &&
                            (StringUtils.isBlank(detailVoTemp.getId()) || !detailVo.getId().equals(detailVoTemp.getId())) && (StringUtils.isBlank(nurseDetailVoTemp.getId()) || !detailVo.getId().equals(nurseDetailVoTemp.getId()))) {
                        returnList.add(detailVo);
                    }else{
                        if(StringUtils.isBlank(detailVo.getTitle())){
                            returnList.add(detailVo);
                        }
                    }
                }
            }
            ((Page<Archive_Detail_Vo>) returnList).setTotal(offsetLimitPage.getTotal());
            return new OffsetLimitPage((Page) returnList);
        }
        return offsetLimitPage;
    }

    @Override
    public int updateCloById(Archive_Detail record) {
        return archiveDetailMapper.updateCloById(record);
    }

    @Override
    public int insertSel(Archive_Detail record) {
        return archiveDetailMapper.insertSel(record);
    }

    @Override
    public Archive_Detail selectByid(String id) {
        return archiveDetailMapper.selectByid(id);
    }

    @Override
    public int deleteByClo(Archive_Detail record) {
        return archiveDetailMapper.deleteByClo(record);
    }

    @Override
    public List<Archive_Detail> selectByColm(Archive_Detail record) {
        return archiveDetailMapper.selectByColm(record);
    }

    @Override
    public List<Archive_Detail> selectByCol(Archive_Detail record) {
        return archiveDetailMapper.selectByCol(record);
    }

    /**
     * 封装合成pdf方法
     * @param response
     * @param arList
     */
    @Override
    public List<String> mulFile2OneById(HttpServletResponse response,List<Archive_Detail> arList) {
        List<String> pdfList =  new ArrayList<>();
        if (arList != null && !arList.isEmpty()) {
            //TODO 抽取首次病程记录
            String strTemp = "首次病程";
            String assortId = "AFB9FBE656D7492C80AEDE6E685A851A";
            //定义护理记录里的首次护理记录排前头
            String nurseTemp = "首次护理";
            String nurseAssortId = "C70E8C427A3648B79BE80798C08F4D12";
            if(!CollectionUtils.isEmpty(arList)) {
                String strPath = "";
                String nursePath = "";
                for (int i = 0; i < arList.size(); i++) {
                    String assortid = arList.get(i).getAssortid();
                    if(StringUtils.isNotBlank(assortid) && assortid.equals(assortId) && arList.get(i).getTitle().contains(strTemp)){
                        strPath = arList.get(i).getPdfPath();
                    }
                    if(StringUtils.isNotBlank(assortid) && assortid.equals(nurseAssortId) && arList.get(i).getTitle().contains(nurseTemp)){
                        nursePath = arList.get(i).getPdfPath();
                    }
                }
                //定义是否添加首次病程
                boolean flag = false;
                boolean nurseFlag = false;
                for (Archive_Detail detail : arList) {
                    String str = detail.getPdfPath();
                    if (StringUtils.isNoneBlank(str)) {
                        //病程分类先添加首次病程
                        if (detail.getAssortid().equals(assortId) && !flag && StringUtils.isNotBlank(strPath)) {
                            pdfList.add(strPath);
                            flag = true;
                        }
                        //护理记录单先添加首次护理
                        if (detail.getAssortid().equals(nurseAssortId) && !nurseFlag && StringUtils.isNotBlank(nursePath)) {
                            pdfList.add(nursePath);
                            nurseFlag = true;
                        }
                        if(!detail.getPdfPath().equals(strPath) && !detail.getPdfPath().equals(nursePath)){
                            pdfList.add(str);
                        }
                    }
                }
            }
        }
        return pdfList;
    }

    @Override
    public void selectPdfPathByIds(HttpServletResponse response,HttpSession session,String detailIds, String masterId,String pdfWater) {
        //获取废除的pdf文件名列表archive_detail
        List<Archive_Detail> arList = archiveDetailMapper.selectPdfPathByIds(masterId,detailIds);
        pdfToSession(response,session,arList,pdfWater);
    }


    private void pdfToSession(HttpServletResponse response,HttpSession session, List<Archive_Detail> arList,String pdfWater){
        List<String> pdfList = mulFile2OneById(response, arList);
        Jpg2PdfUtil jpg2PdfUtil = Jpg2PdfUtil.getInstance();
        jpg2PdfUtil.mulFile2One(response, pdfList, pdfWater);
        //存至session
        //session.setAttribute("showRecord",pdfList);
    }

    @Override
    public void selectByIdStr(HttpServletResponse response, HttpSession session,Archive_Detail detail,String pdfWater) {
        List<Archive_Detail> arList = archiveDetailMapper.selectByIdStr(detail);
        pdfToSession(response,session,arList,pdfWater);
    }

    @Override
    public void selectByIdStrFont(HttpServletResponse response, HttpSession session,Archive_Detail detail) {
        List<Archive_Detail> arList = archiveDetailMapper.selectByIdStr(detail);
        List<String> pdfList = mulFile2OneById(response, arList);
        //存至session
        session.setAttribute("showRecord",pdfList);
    }

    @Override
    public void showPdf(HttpServletResponse response, HttpSession session, String pdfWater) {
        List<String> pdfList = (List<String>)session.getAttribute("showRecord");
        if(!CollectionUtils.isEmpty(pdfList)) {
            Jpg2PdfUtil jpg2PdfUtil = Jpg2PdfUtil.getInstance();
            jpg2PdfUtil.mulFile2One(response, pdfList, pdfWater);
        }
        //删除缓存`                                                                 `   `
        session.removeAttribute("showRecord");
    }

    @Override
    public List<Archive_Detail> getSourceByMasterId(String masterid) {
        return archiveDetailMapper.getSourceByMasterId(masterid);
    }

    @Override
    public List<Archive_Detail> selectTypeTreeByPatientIdAndAssortIds(String patientId, String assortIds) {
        return archiveDetailMapper.selectTypeTreeByPatientIdAndAssortIds(patientId, assortIds);
    }

    @Override
    public String selectIsScan(String ids) {
        //判断是否需要判断纸质病历已扫描
        Emr_Commom_Set commomSet = commomSetMapper.selectByPrimaryKey(1);
        if(commomSet.getInt1() == 1){
            StringBuilder idsStr = new StringBuilder();
            //组织已经扫描上传的MasterId
            List<Archive_Detail> list =  archiveDetailMapper.selectIsScan(ids,2);
            ids = "";
            if(!CollectionUtils.isEmpty(list)){
                for (Archive_Detail obj: list ) {
                    String masterid = obj.getMasterid();
                    if(StringUtils.isBlank(idsStr)){
                        idsStr.append(masterid);
                    }else{
                        idsStr.append(",").append(masterid);
                    }
                }
                ids = idsStr.toString();
            }
        }
        return ids;
    }
}
 

