/**
 * Copyright (C), 2015-2019
 * Author:   HJL
 * Date:     2019/7/19 15:39
 * Description:患者住院信息主索引管理
 */
package com.emr.service.ipml;

import com.alibaba.fastjson.JSONObject;
import com.emr.dao.*;
import com.emr.entity.*;
import com.emr.service.Archive_MasterService;
import com.emr.service.Emr_Fault_DetailService;
import com.emr.util.ExceptionPrintUtil;
import com.emr.util.HttpClientUtils;
import com.emr.util.Msg;
import com.emr.util.OracleConnect;
import com.emr.vo.CommomVo;
import com.emr.vo.User;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.lang3.StringUtils;
import org.apache.log4j.Logger;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.util.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author zwh
 * */
@Service
@Transactional
public class Archive_MasterServiceImpl implements Archive_MasterService {
    private static Logger log = Logger.getLogger("myLog");
    @Autowired
    private Archive_MasterMapper archiveMasterMapper;
    @Autowired
    private Emr_Fault_DetailService emrFaultDetailService;
    @Autowired
    private ArchiveFlowRoleMapper flowRoleMapper;
    @Autowired
    private ArchiveFlowInfoService flowInfoService;
    @Autowired
    private Archive_DetailMapper detailMapper;
    @Autowired
    private Zd_AssortMapper assortMapper;
    @Autowired
    private Emr_DictionaryMapper dictionaryMapper;
    @Autowired
    private PaperPostionService paperPostionService;
    @Value("${POWER_URLHEAD}")
    private String POWER_URLHEAD;
    @Autowired
    private CommomService commomService;
    @Autowired
    private TUuPrintMapper tUuPrintMapper;
    @Autowired
    private  Archive_MasterMapper archive_masterMapper;
    /**
     * 在院列表
     * */
    @Override
    public OffsetLimitPage selectByCol(Archive_Master_Vo archiveMasterVo, Integer offset, Integer limit) {
        PageHelper.offsetPage(offset, limit);
        List<Archive_Master> list= archiveMasterMapper.selectByCol(archiveMasterVo);
        return new OffsetLimitPage((Page) list);
    }

    /**
     * 出院列表
     * */
    @Override
    public OffsetLimitPage selectByColumn(Archive_Master_Vo archiveMasterVo, Integer offset, Integer limit,HttpServletRequest request) {
        //设置查询条件审核角色
        setInfoId(archiveMasterVo);
        //设置登录用户
        Power_User user = (Power_User)request.getSession().getAttribute("CURRENT_USER");
        if(user.getRoleId() != -100 && user.getRoleId() != 0 ){
            archiveMasterVo.setUserName(user.getUserName());

        }

        //查询
        List<Archive_Master_Vo> list = selectByColumn(offset, limit,archiveMasterVo,request);
        return new OffsetLimitPage((Page) list);
    }

    @Override
    public OffsetLimitPage selectMasterByColumn(Archive_Master_Vo archiveMasterVo, Integer offset, Integer limit,HttpServletRequest request) {
        //设置查询条件审核角色
        //setInfoId(archiveMasterVo);
        //设置登录用户
        /*Power_User user = (Power_User)request.getSession().getAttribute("CURRENT_USER");
        //查该登录工号绑定的科室下面的患者，且只能查在院患者
        String requestUrl =POWER_URLHEAD + "/ssoLogin/getUser?userName="+user.getUserName();
        JSONObject obj = HttpClientUtils.httpGet(requestUrl);
        if(obj!=null) {
            archiveMasterVo.setDeptName(obj.getString("deptCode"));
        }*/
        //查询
        List<Archive_Master_Vo> list = selectByColumn(offset, limit,archiveMasterVo,request);
        return new OffsetLimitPage((Page) list);
    }

    @Override
    public List<Archive_Master> getDoctorInCharge(Archive_Master record,HttpServletRequest request)throws Exception {
        //设置登录用户
        Power_User user = (Power_User)request.getSession().getAttribute("CURRENT_USER");
        //查该登录工号绑定的科室下面的患者，且只能查在院患者
        String requestUrl =POWER_URLHEAD + "/ssoLogin/getUser?userName="+user.getUserName();
        //List<User> userList = commomService.getUserList();
        JSONObject obj = HttpClientUtils.httpGet(requestUrl);
        if(obj!=null) {
            record.setDeptName(obj.getString("deptCode"));
        }
        List<Archive_Master> returnList=archiveMasterMapper.getDoctorInCharge(record);
        /*//替换姓名
        for (Archive_Master archiveMasterList : returnList) {
            if (!CollectionUtils.isEmpty(userList)) {
                //转换姓名
                String userName = archiveMasterList.getDoctorInCharge();
                for (User userDetail : userList) {
                    if (StringUtils.isNotBlank(userName) && userName.equals(userDetail.getUserName())) {
                        archiveMasterList.setDoctorInCharge(userDetail.getName());
                        break;
                    }
                }
            }
        }*/
        return archiveMasterMapper.getDoctorInCharge(record);
    }

    /**
     * //循环遍历list集合判断是否完整，完整isCheck赋值1，且批量更新LockInfo字段为完整
     * @param list
     */
    private void updateMasterBySuccess(List<Archive_Master_Vo> list) {
        //list集合的分组分类集合
        if(!CollectionUtils.isEmpty(list)){
            //获取资料完整的masterIds
            String masterIds = getMasterIdsByList(list);
            Archive_Master_Vo masterVo = new Archive_Master_Vo();
            masterVo.setMasterIds(masterIds);
            masterVo.setIsSuccess("1");
            //查询自定义的完整性的masterIds集合
            List<Archive_Master_Vo> selectCompleteMasterIds = archiveMasterMapper.selectCompleteMasterIds(masterVo);
            if(!CollectionUtils.isEmpty(selectCompleteMasterIds)) {
                //将暂时赋值完整的isSuccess = 1
                setIsSuccess(selectCompleteMasterIds,list);
                //获取mastertIds集合
                masterIds = getMasterIdsByList(selectCompleteMasterIds);
                List<Archive_Detail> selectNotInspectionList = null;
                List<Archive_Detail> repeatRecordFileForMasterIds = null;
                if (StringUtils.isNotBlank(masterIds)) {
                    //查询缺少检验检查报告masterId集合
                    selectNotInspectionList = detailMapper.selectNotInspectionList(masterIds);
                    //查询有重复采集文件的masterIds集合
                    repeatRecordFileForMasterIds = detailMapper.selectRepeatRecordFileForMasterIds(masterIds);
                }
                //循环遍历list集合判断是否完整，完整isCheck赋值1，且批量更新LockInfo字段为完整
                //定义通过完整性校验的masterIds
                StringBuilder masterIdsTemp = new StringBuilder();
                for(Archive_Master_Vo master : list) {
                    //定义检验检查报告分类是完整的
                    boolean inspectionFlag = false;
                    //判断不存在缺少校验检查分类
                    if (!CollectionUtils.isEmpty(selectNotInspectionList)) {
                        for (Archive_Detail detail : selectNotInspectionList) {
                            if (master.getId().equals(detail.getMasterid())) {
                                inspectionFlag = true;
                                break;
                            }
                        }
                    }
                    //判断不存在重复文件
                    boolean repeatFileFlag = true;
                    if (!CollectionUtils.isEmpty(repeatRecordFileForMasterIds)) {
                        for (Archive_Detail detail : repeatRecordFileForMasterIds) {
                            if (master.getId().equals(detail.getMasterid())) {
                                repeatFileFlag = false;
                                break;
                            }
                        }
                    }
                    //检验检查报告分类完整才判断需要校验的分类是否完整
                    if (inspectionFlag || !repeatFileFlag) {
                        //list的isSuccess字段值赋值1
                        master.setIsSuccess("0");
                    }
                    //不完整的批量更新为完整
                    if (!"完整".equals(master.getLockinfo()) && "1".equals(master.getIsSuccess())) {
                        if (StringUtils.isNotBlank(masterIdsTemp)) {
                            masterIdsTemp.append(",");
                        }
                        masterIdsTemp.append("'").append(master.getId()).append("'");
                    }
                }
                if (StringUtils.isNotBlank(masterIdsTemp)) {
                    archiveMasterMapper.updateLockInfoByMasterId(masterIdsTemp.toString(), "完整");
                }
            }
        }
    }

    /**
     * 暂时设置完整isSuccess = 1
     * @param selectCompleteMasterIds
     * @param list
     */
    private void setIsSuccess(List<Archive_Master_Vo> selectCompleteMasterIds, List<Archive_Master_Vo> list) {
        for(Archive_Master_Vo vo : list){
            for(Archive_Master_Vo vo1 : selectCompleteMasterIds){
                if(vo.getId().equals(vo1.getId())){
                    vo.setIsSuccess("1");
                    break;
                }
            }
        }
    }

    /**
     * 组织masterIds集合
     * @param list
     * @return
     */
    private String getMasterIdsByList(List<Archive_Master_Vo> list) {
        StringBuilder masterIds = new StringBuilder();
        for (Archive_Master_Vo vo : list) {
            if (StringUtils.isNotBlank(masterIds)) {
                masterIds.append(",");
            }
            masterIds.append("'").append(vo.getId()).append("'");
        }
        return masterIds.toString();
    }

    /**
     * 未归档列表
     * */
    @Override
    public OffsetLimitPage selectByUnfile(Archive_Master_Vo archiveMasterVo, Integer offset, Integer limit) {
        PageHelper.offsetPage(offset, limit);
        List<Archive_Master_Vo> list = selectByUnfile(archiveMasterVo);
        return new OffsetLimitPage((Page) list);
    }

    /**
     * 终审列表
     * */
    @Override
    public OffsetLimitPage selectByLast(Archive_Master_Vo archiveMasterVo, Integer offset, Integer limit) {
        PageHelper.offsetPage(offset, limit);
        List<Archive_Master_Vo> list = selectLastVerifyList(archiveMasterVo);
        return new OffsetLimitPage((Page) list);
    }

    /**
     * 查询终审页面列表
     * */
    @Override
    public List<Archive_Master_Vo> selectLastVerifyList(Archive_Master_Vo archiveMasterVo){
        List<Archive_Master_Vo> list = archiveMasterMapper.selectByLast(archiveMasterVo);
        if(null != list && !list.isEmpty()){
            //查询科室集合
            Emr_Dictionary dictionary = new Emr_Dictionary();
            dictionary.setTypecode("dept_code");
            List<Emr_Dictionary> dictionaries = dictionaryMapper.dicByTypeCode(dictionary);
            for (Archive_Master_Vo masterVo : list) {
                //科室名称编码转名称
                deptNameCode2Name(dictionaries, masterVo);
                //状态均为护士长已质控，取出一个对象，转换中文状态
                Integer status = Integer.valueOf(masterVo.getArchivestate());
                if(status.equals(EnumVerify.DocState.HeadNurseCheck.getCode())){
                    masterVo.setStatus(EnumVerify.DocState.HeadNurseCheck.getName());
                }else if(status.equals(EnumVerify.DocState.CallBacked.getCode())){
                    masterVo.setStatus(EnumVerify.DocState.CallBacked.getName());
                }
            }
        }
        return list;
    }

    @Override
    public List<Archive_Master> loadDoctorInCharge(String deptName) {
        return archiveMasterMapper.loadDoctorInCharge(deptName);
    }

    @Override
    public List<Archive_Master> loadOverTimeDoctorInCharge() {
        return archiveMasterMapper.loadOverTimeDoctorInCharge();
    }

    /**
     * 已归档
     * 病案终审退回页面
     * */
    @Override
    public OffsetLimitPage selectFiled(Archive_Master_Vo master,Integer offset, Integer limit) {
        PageHelper.offsetPage(offset, limit);
        List<Archive_Master_Vo> list= archiveMasterMapper.selectFiled(master);
        return new OffsetLimitPage((Page) list);
    }

    @Override
    public List<Archive_Master> selectByCol(Archive_Master_Vo archiveMasterVo) {
        return archiveMasterMapper.selectByCol(archiveMasterVo);
    }

    @Override
    public List<Archive_Master_Vo> selectByColumn(Integer offset, Integer limit,Archive_Master_Vo archiveMasterVo,HttpServletRequest request) {
        //分是否按完整性查询
        List<Archive_Master_Vo> list = null;
        if(null != offset && null != limit) {
            PageHelper.offsetPage(offset, limit);
        }
        list = archiveMasterMapper.selectByColumn(archiveMasterVo);

        //组织完整性字段并完整的更新archive_master表的完整性字段值为完整LockInfo='完整'
        //updateMasterBySuccess(list);
        //转换科室与赋值超期天数
       // selectByColumnChange(list);
        //根据状态code转换状态及判断显示按钮
        /*if(null != list && !list.isEmpty()){
            //查询流转角色集合
            List<ArchiveFlowRole> flowRoles = flowRoleMapper.selectAll();
            //计算属于哪种审核角色
            Msg role = getRole();
            if(null != role) {
                for (Archive_Master_Vo masterVo : list) {
                    String archivestate = masterVo.getArchivestate();
                    if (StringUtils.isNotBlank(archivestate)) {
                        //转换中文状态
                        boolean numeric = isNumeric(archivestate);
                        if (numeric) {
                            Integer status = Integer.valueOf(archivestate);
                            //组织已经审核的状态
                            archivestate = EnumVerify.DocState.GetStepName(status, role.getCode());
                            //组织未审核的状态
                            String currentArchivestate = EnumVerify.DocState.GetCurrentStepName(status, role.getCode());
                            masterVo.setStatus(archivestate);
                            masterVo.setCurrentStatus(currentArchivestate);
                            //根据角色
                            //获取审核按钮和退回按钮集合
                            String btns = getBtns(status, flowRoles, masterVo.getId());
                            masterVo.setBtns(btns);
                        }
                    }
                }
            }
        }*/
        Emr_Dictionary dictionary = new Emr_Dictionary();
        dictionary.setTypecode("dept_code");
        List<Emr_Dictionary> dictionaries = dictionaryMapper.dicByTypeCode(dictionary);
        for (Archive_Master_Vo masterVo : list) {
            //科室名称编码转名称
            deptNameCode2Name(dictionaries, masterVo);
            //状态均为护士长已质控，取出一个对象，转换中文状态
            Integer status = Integer.valueOf(masterVo.getArchivestate());
            if(status.equals(EnumVerify.DocState.HeadNurseCheck.getCode())){
                masterVo.setStatus(EnumVerify.DocState.HeadNurseCheck.getName());
            }else if(status.equals(EnumVerify.DocState.CallBacked.getCode())){
                masterVo.setStatus(EnumVerify.DocState.CallBacked.getName());
            }
        }
        if (null != list && !list.isEmpty()) {
            List<ArchiveFlowRole> flowRoles = this.flowRoleMapper.selectAll();
            Msg role = this.getRole();
            if (null != role) {
                Iterator var8 = list.iterator();

                while(var8.hasNext()) {
                    Archive_Master_Vo masterVo = (Archive_Master_Vo)var8.next();
                    String archivestate = masterVo.getArchivestate();
                    if (StringUtils.isNotBlank(archivestate)) {
                        boolean numeric = this.isNumeric(archivestate);
                        if (numeric) {
                            Integer status = Integer.valueOf(archivestate);
                            archivestate = EnumVerify.DocState.GetStepName(status, role.getCode());
                            String currentArchivestate = EnumVerify.DocState.GetCurrentStepName(status, role.getCode());
                            masterVo.setStatus(archivestate);
                            masterVo.setCurrentStatus(currentArchivestate);
                            String btns = this.getBtns(status, flowRoles, masterVo.getId());
                            masterVo.setBtns(btns);
                        }
                    }
                }
            }
        }
        return list;
    }

    /**
     * 带完整性查询的出院记录查询
     * @param archiveMasterVo
     * @return
     */
    private List<Archive_Master_Vo> selectBeHospitalByIsSuccess(Integer offset,Integer limit,Archive_Master_Vo archiveMasterVo) {
        if(null != offset && null != limit) {
            PageHelper.offsetPage(offset, limit);
        }
        return archiveMasterMapper.selectCompleteMasterIds(archiveMasterVo);
    }

    /**
     * 设置审核角色
     * @param archiveMasterVo
     */
    private void setInfoId(Archive_Master_Vo archiveMasterVo) {
        String status = archiveMasterVo.getStatus();
        //取最大值
        String archivestate = archiveMasterVo.getArchivestate();
        if(StringUtils.isNotBlank(status) && StringUtils.isNotBlank(archivestate)){
            if ("1".equals(archivestate)){
                //已审核，取最大
                Integer maxStatus = 0;
                String[] statusArr = status.split(",");
                for (String statu : statusArr) {
                    if (StringUtils.isNotBlank(statu)) {
                        Integer integer = Integer.valueOf(statu);
                        if (maxStatus < integer) {
                            //重新赋值
                            maxStatus = integer;
                        }
                    }
                }
                archiveMasterVo.setStatus(maxStatus+"");
            }else{
                //待审核，取带逗号的字符串的数
                archiveMasterVo.setStatus(status);
            }
        }
    }

    /**
     * 出院浏览转换科室中文、查询天数
     *
     * @param list
     */
    private void selectByColumnChange(List<Archive_Master_Vo> list) {
        if(!CollectionUtils.isEmpty(list)){
            //查询科室集合
            Emr_Dictionary dictionary = new Emr_Dictionary();
            dictionary.setTypecode("dept_code");
            List<Emr_Dictionary> dictionaries = dictionaryMapper.dicByTypeCode(dictionary);
            //查询出院天数
            List<Archive_Master_Vo> selectDays = new ArrayList<>();
            if(list.size() > 200){
                selectDays = archiveMasterMapper.selectDaysByMasterIds(null);
            }else{
               selectDays = archiveMasterMapper.selectDaysByMasterIds(list);
            }

            for (Archive_Master_Vo vo:list) {
                //科室名称编码转名称
                deptNameCode2Name(dictionaries, vo);
                //赋值出院天数
                if(!CollectionUtils.isEmpty(selectDays)){
                    for(Archive_Master_Vo days : selectDays){
                        if(days.getId().equals(vo.getId())){
                            vo.setDays(days.getDays());
                            break;
                        }
                    }
                }
            }
        }
    }

    /**
     * 科室名称编码转名称
     * @param dictionaries 科室字典列表
     * @param vo 患者记录对象
     */
    private void deptNameCode2Name(List<Emr_Dictionary> dictionaries, Archive_Master_Vo vo) {
        if(!CollectionUtils.isEmpty(dictionaries)) {
            //在院科室
            String deptAdmissionTo = vo.getDeptAdmissionTo();
            //出院科室
            String deptName = vo.getDeptName();
            //住院科室
            for (Emr_Dictionary dictionaryTemp : dictionaries) {
                String name = dictionaryTemp.getName();
                if (StringUtils.isNotBlank(deptAdmissionTo) && deptAdmissionTo.equals(dictionaryTemp.getCode())) {
                    vo.setDeptAdmissionTo(name);
                }
                if (StringUtils.isNotBlank(deptName) && deptName.equals(dictionaryTemp.getCode())) {
                    vo.setDeptName(name);
                }
            }
        }
    }

    //验证字符串是否是数字
    @Override
    public boolean isNumeric(String str){
        for (int i = 0; i < str.length(); i++){
            if (!Character.isDigit(str.charAt(i))){
                return false;
            }
        }
        return true;
    }
    /**
     * 处理已审批查询时的节点id,且是多个的
     * @param archiveMasterVo
     * @return
     */
    private void setMaxRoleId(Archive_Master_Vo archiveMasterVo){
        String status = archiveMasterVo.getStatus();
        //取最大值
        String archivestate = archiveMasterVo.getArchivestate();
        if(StringUtils.isNotBlank(status) && StringUtils.isNotBlank(archivestate)){
            //查询已审批才处理
            if("1".equals(archivestate) && status.contains(",")) {
                Integer maxStatus = 0;
                String[] statusArr = status.split(",");
                for (String statu : statusArr) {
                    if (StringUtils.isNotBlank(statu)) {
                        Integer integer = Integer.valueOf(statu);
                        if (maxStatus < integer) {
                            //重新赋值
                            maxStatus = integer;
                        }
                    }
                }
                //设置角色index
                archiveMasterVo.setStatus(maxStatus+"");
            }
        }
    }

    @Override
    public List<Archive_Master_Vo> selectByUnfile(Archive_Master_Vo archiveMasterVo) {
        List<Archive_Master_Vo> list = archiveMasterMapper.selectByUnfile(archiveMasterVo);
        //根据状态code转换状态及判断显示按钮
        if(null != list && !list.isEmpty()){
            //计算属于哪种审核角色
            Msg role = getRole();
            if(null != role) {
                for (Archive_Master_Vo master_vo : list) {
                    String archivestate = master_vo.getArchivestate();
                    String dischargeDateTime = master_vo.getDischargeDateTime();
                    //年份是1801定义为在院
                    //定义是否是在院病历
                    boolean flag = false;
                    if (StringUtils.isNotBlank(dischargeDateTime) && dischargeDateTime.length() >= 4) {
                        String year = dischargeDateTime.substring(0, 4);
                        if ("1801".equals(year)) {
                            flag = true;
                            master_vo.setStatus("在院");
                        }
                    }
                    if (StringUtils.isNotBlank(archivestate) && !flag) {
                        //转换中文状态
                        Integer status = Integer.valueOf(archivestate);
                        archivestate = EnumVerify.DocState.GetStepName(status, role.getCode());
                        master_vo.setStatus(archivestate);
                    }
                }
            }
        }
        return list;
    }

    /**
     * 获取审核按钮和退回按钮集合
     * */
    private String getBtns(Integer status,List<ArchiveFlowRole> roles,String id){
        String btns = "";
        //获取用户功能权限信息
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        Power_User user = (Power_User) request.getSession().getAttribute("CURRENT_USER");
        Set<String> menus = user.getMenus();
        if(null != menus && !menus.isEmpty()) {
            //医生提交权限标识
            String doctorSumbit = "/beHosp/doctorSumbit";
            //医生质控员质控权限标识
            String doctorCheckSumbit = "/beHosp/doctorCheckSumbit";
            //科主任审核权限标识
            String directorCheckSumbit = "/beHosp/directorCheckSumbit";
            //护士提交权限标识
            String nurseSumbit = "/beHosp/nurseSumbit";
            //护士质控员质控权限标识
            String nurseCheckSumbit = "/beHosp/nurseCheckSumbit";
            //护士长审核权限标识
            String headNurseCheck = "/beHosp/headNurseCheck";
            //退回权限标识
            String backSumbit = "/beHosp/backSumbit";
            //医生提交权限标识
            int doctorSumbitPower = 0;
            //医生质控员质控权限标识
            int doctorCheckSumbitPower = 0;
            //科主任审核权限标识
            int directorCheckSumbitPower = 0;
            //护士提交权限标识
            int nurseSumbitPower = 0;
            //护士质控员质控权限标识
            int nurseCheckSumbitPower = 0;
            //护士长审核权限标识
            int headNurseCheckPower = 0;
            //退回权限标识
            int backPower = 0;
            //遍历，判断拥有权限，拥有护士长权限立即返回4，拥有医生支线赋值doctorRole=1,拥有护士支线赋值nurseRole=1
            for (String menu : menus) {
                if (StringUtils.isNotBlank(menu)) {
                    if (menu.equals(doctorSumbit+"")) {
                        doctorSumbitPower = 1;
                    }
                    if (menu.equals(doctorCheckSumbit+"")) {
                        doctorCheckSumbitPower = 1;
                    }
                    if (menu.equals(directorCheckSumbit+"")) {
                        directorCheckSumbitPower = 1;
                    }
                    if (menu.equals(nurseSumbit+"")) {
                        nurseSumbitPower = 1;
                    }
                    if (menu.equals(nurseCheckSumbit+"")) {
                        nurseCheckSumbitPower = 1;
                    }
                    if (menu.equals(headNurseCheck+"")) {
                        headNurseCheckPower = 1;
                    }
                    if (menu.equals(backSumbit+"")) {
                        backPower = 1;
                    }
                }
            }
            //判断按钮
            //定义按钮
            String headNurseSumbitName = "护士长审核";
            String doctorSumbitName = "医生提交";
            String doctorCheckSumbitName = "医生质控员质控";
            String directorCheckSumbitName = "科主任审核";
            String nurseSumbitName = "护士提交";
            String nurseCheckSumbitName = "护士质控员质控";
            //定义医生提交是否有效
            int doctorSubmitEffective = 0;
            //定义医生质控员是否有效
            int dortorCheckEffective = 0;
            //定义科主任审核是否有效
            int directorCheckEffective = 0;
            //定义护士提交是否有效
            int nurseSubmitEffective = 0;
            //定义护士质控员质控是否有效
            int nurseCheckEffective = 0;
            //定义护士长质控是否有效
            int headNurseCheckEffective = 0;
            for (ArchiveFlowRole flowRole : roles) {
                if(flowRole.getCode().equals(EnumVerify.DocState.HeadNurseCheck.getCode()+"")){
                    headNurseSumbitName = flowRole.getSumbitName();
                    headNurseCheckEffective = flowRole.getEffective();
                }else if(flowRole.getCode().equals(EnumVerify.DocState.DortorSubmit.getCode()+"")){
                    doctorSumbitName = flowRole.getSumbitName();
                    doctorSubmitEffective = flowRole.getEffective();
                }else if(flowRole.getCode().equals(EnumVerify.DocState.DortorCheck.getCode()+"")){
                    doctorCheckSumbitName = flowRole.getSumbitName();
                    dortorCheckEffective = flowRole.getEffective();
                }else if(flowRole.getCode().equals(EnumVerify.DocState.DirectorCheck.getCode()+"")){
                    directorCheckSumbitName = flowRole.getSumbitName();
                    directorCheckEffective = flowRole.getEffective();
                }else if(flowRole.getCode().equals(EnumVerify.DocState.NurseSubmit.getCode()+"")){
                    nurseSumbitName = flowRole.getSumbitName();
                    nurseSubmitEffective = flowRole.getEffective();
                }else if(flowRole.getCode().equals(EnumVerify.DocState.NurseCheck.getCode()+"")){
                    nurseCheckSumbitName = flowRole.getSumbitName();
                    nurseCheckEffective = flowRole.getEffective();
                }
            }
            //判断组织审核按钮
            //审核按钮参数1:角色编号1或2或4。2:当前code。3.下个节点code。4:masterId
            //退回按钮参数1:当前code。2:下个节点code。3:masterId
            //护士长
            String masterId = '"' + id + '"';
            //定义下个节点code
            int targetCode = 0;
            if(!EnumVerify.DocState.HasCurrentState(status,EnumVerify.DocState.HeadNurseCheck)){
                //判断是否护士长审核
                boolean headNurseCheckFlag = getHeadNurseCheckFlag(status,roles);
                if (headNurseCheckFlag) {
                    if (headNurseCheckPower == 1 && headNurseCheckEffective == 1) {
                        targetCode = EnumVerify.DocState.HeadNurseCheck.getCode();
                        btns += "<button type='button' class='btn btn-primary btn-sm' onclick='verifyInfo(" + EnumVerify.Checker.Generate.getCode() + "," + status + "," + targetCode +","+masterId+")' title="+headNurseSumbitName+">"+headNurseSumbitName+"</button>";

                    }
                } else {
                    //没有医生提交
                    if (!EnumVerify.DocState.HasCurrentState(status, EnumVerify.DocState.DortorSubmit)) {
                        if (doctorSumbitPower == 1 && doctorSubmitEffective == 1) {
                            targetCode = EnumVerify.DocState.DortorSubmit.getCode();
                            btns += "<button type='button' class='btn btn-primary btn-sm' onclick='verifyInfo(" + EnumVerify.Checker.Dortor.getCode() + "," + status + "," + targetCode+ "," + masterId +")' title="+doctorSumbitName+">"+doctorSumbitName+"</button>";
                        }
                    }
                    //没有医生质控员提交
                    if (!EnumVerify.DocState.HasCurrentState(status, EnumVerify.DocState.DortorCheck) && EnumVerify.DocState.HasCurrentState(status, EnumVerify.DocState.DortorSubmit) || doctorSubmitEffective == 0) {
                        if (doctorCheckSumbitPower == 1 && dortorCheckEffective == 1) {
                            targetCode = EnumVerify.DocState.DortorCheck.getCode();
                            btns += "<button type='button' class='btn btn-primary btn-sm' onclick='verifyInfo(" + EnumVerify.Checker.Dortor.getCode() + ","  + status + "," + targetCode+","+masterId+")' title="+doctorCheckSumbitName+">"+doctorCheckSumbitName+"</button>";
                        }
                    }
                    //没有科主任审核
                    if (!EnumVerify.DocState.HasCurrentState(status, EnumVerify.DocState.DirectorCheck) && EnumVerify.DocState.HasCurrentState(status, EnumVerify.DocState.DortorCheck) || (dortorCheckEffective == 0 && doctorSubmitEffective == 0) || (dortorCheckEffective == 0 && EnumVerify.DocState.HasCurrentState(status, EnumVerify.DocState.DortorSubmit))) {
                        if (directorCheckSumbitPower == 1 && directorCheckEffective == 1) {
                            targetCode = EnumVerify.DocState.DirectorCheck.getCode();
                            btns += "<button type='button' class='btn btn-primary btn-sm' onclick='verifyInfo(" + EnumVerify.Checker.Dortor.getCode() + ","  + status + "," + targetCode+","+masterId+")' title="+directorCheckSumbitName+">"+directorCheckSumbitName+"</button>";
                        }
                    }
                    //没有护士提交
                    if (!EnumVerify.DocState.HasCurrentState(status, EnumVerify.DocState.NurseSubmit)) {
                        if (nurseSumbitPower == 1 && nurseSubmitEffective == 1) {
                            targetCode = EnumVerify.DocState.NurseSubmit.getCode();
                            btns += "<button type='button' class='btn btn-primary btn-sm' onclick='verifyInfo(" + EnumVerify.Checker.Nurse.getCode() + "," + status + "," + targetCode+","+masterId+")' title="+nurseSumbitName+">"+nurseSumbitName+"</button>";
                        }
                        //没有护士质控员提交
                    }
                    if (!EnumVerify.DocState.HasCurrentState(status, EnumVerify.DocState.NurseCheck) && EnumVerify.DocState.HasCurrentState(status, EnumVerify.DocState.NurseSubmit) || nurseSubmitEffective == 0) {
                        if (nurseCheckSumbitPower == 1 && nurseCheckEffective == 1) {
                            targetCode = EnumVerify.DocState.NurseCheck.getCode();
                            btns += "<button type='button' class='btn btn-primary btn-sm' onclick='verifyInfo(" + EnumVerify.Checker.Nurse.getCode() + ","  + status + "," + targetCode+","+masterId+")' title="+nurseCheckSumbitName+">"+nurseCheckSumbitName+"</button>";
                        }
                    }
                }
                //退回按钮
                if(StringUtils.isNotBlank(btns) && backPower == 1){
                    btns += "<button type='button' class='btn btn-warning btn-sm ' onclick='returnInfo("+status + "," + targetCode + "," + masterId + ")' title='退回'>退回</button>";
                }
            }
        }
        return btns;
    }
    /**
     * 计算是否属于护士长审核
     * */
    private boolean getHeadNurseCheckFlag(int status,List<ArchiveFlowRole> roles){
        boolean flag = true;
        List<ArchiveFlowRole> effectiveRoles = new ArrayList<>();
        //遍历查询匹配医生和护士支线最后节点均已提交
        for (ArchiveFlowRole role : roles) {
            //获取步骤
            Integer stepIndex = role.getStepIndex();
            //获取有效
            Short effective = role.getEffective();
            //获取有效且护士长审核前的节点且不包含开始节点
            if (null != stepIndex && stepIndex < 4 && stepIndex > 0 && effective == 1) {
                effectiveRoles.add(role);
            }
        }
        if(!effectiveRoles.isEmpty()){
            for(ArchiveFlowRole role : effectiveRoles){
                //获取节点code
                String code = role.getCode();
                //转换类型
                Integer integerCode = Integer.valueOf(code);
                //其中有一个不包含直接返回false
                if((status & integerCode) == 0){
                    return false;
                }
            }
        }
        return flag;
    }

    @Override
    public int updateByClo(Archive_Master record) {
        return archiveMasterMapper.updateByClo(record);
    }

    @Override
    public int updateById(Archive_Master record) {
        return archiveMasterMapper.updateById(record);
    }

    @Override
    public Archive_Master selectByPrimaryKey(String id) {
        return archiveMasterMapper.selectByPrimaryKey(id);
    }


    /**
     * 插入第三方oracle库
     * @param flowInfo
     */
    private void backInsertOracle(Integer code,ArchiveFlowInfo flowInfo){
        //查询是否存在记录
        String masterId = flowInfo.getMasterId();
        String selectSql = "SELECT count(1) FROM ARCHIVE_FLOW_INFO where master_id = '"+masterId+"'";
        List<String> counts = null;
        try {
            counts = OracleConnect.selectList(selectSql);
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException("审核时查询oracle记录失败");
        }
        if(code == 1 || !"0".equals(counts.get(0))) {
            Archive_Master master = archiveMasterMapper.selectByPrimaryKey(masterId);
            String patientId = master.getPatientId();
            String sql = "insert all into archive_flow_info (" +
                    "master_id, " +
                    "step_index," +
                    "start_step_id, " +
                    "start_step, " +
                    "sumbit_name," +
                    "target_step_id, " +
                    "target_step, " +
                    "oper_role," +
                    "use_seconds, " +
                    "remark, " +
                    "user_name," +
                    "sumbit_time, " +
                    "createtime," +
                    "jzh," +
                    "is_available," +
                    "return," +
                    "return_time" +
                    ")" +
                    "values ("
                    + "'" + flowInfo.getMasterId() + "',"
                    + flowInfo.getStepIndex() + ","
                    + flowInfo.getStartStepId() + ","
                    + "'" + flowInfo.getStartStep() + "',"
                    + "'" + flowInfo.getSumbitName() + "',"
                    + flowInfo.getTargetStepId() + ","
                    + "'" + flowInfo.getTargetStep() + "',"
                    + flowInfo.getOperRole() + ","
                    + "'" + flowInfo.getUseSeconds() + "',"
                    + "'" + flowInfo.getRemark() + "',"
                    + "'" + flowInfo.getUserName() + "',"
                    + "SYSDATE,"
                    + "SYSDATE,"
                    + "'" + patientId + "',"
                    + "'" + "否" + "',"
                    + "'" + "是" + "',"
                    + "SYSDATE"
                    + ") select * from dual";
            try {
                OracleConnect.insert(sql);
            } catch (Exception e) {
                e.printStackTrace();
                throw new RuntimeException("审核信息插入oracle失败");
            }
        }
    }

    /**
     * 退回，修改oracle信息 将该支线的所有可用都改为'否',并修改是否退回为''和退回时间
     * @param roleIndex
     * @param archiveDetailId
     */
    private void updateOracle(String roleIndex, String archiveDetailId) {
        //修改语句
        String sql = "update ARCHIVE_FLOW_INFO set is_available = '否' where master_id = '"+archiveDetailId+"'";
        if(StringUtils.isNotBlank(roleIndex)){
            //修改单条支线
            sql += " and OPER_ROLE in ("+roleIndex+",4)";
        }
        try {
            OracleConnect.update(sql);
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException("退回修改oracle失败");
        }
    }

    /**
     * 退回操作
     * */
    @Override
    public int updateReturn(Emr_Fault_Detail_Vo emrFaultDetail) throws Exception{
        //修改病案归档状态：复审退回
        Archive_Master archiveMaster = new Archive_Master();
        // 从session获取用户名
        String username = (String) SecurityUtils.getSubject().getSession().getAttribute("userSession");//获取前面登录的用户名
        //设置回退给角色信息
        String returntoRole = emrFaultDetail.getReturntoRole();
        log.info(username+"将病案:"+emrFaultDetail.getArchiveDetailId()+"退回给"+returntoRole);
        if(StringUtils.isNotBlank(emrFaultDetail.getStatus())) {
            int status = Integer.valueOf(emrFaultDetail.getStatus());
            //计算状态code
            //查询审核角色集合，根据有效进行计算状态code
            List<ArchiveFlowRole> archiveFlowRoles = flowRoleMapper.selectAll();
            //定义masterId
            String masterId = emrFaultDetail.getArchiveDetailId();
            //定义下个节点code
            String targetCode = "";
            //为空为病案室退回
            if(null == emrFaultDetail.getTargetCode()){
                targetCode = EnumVerify.DocState.MedicalRoomCheck.getCode() + "";
            }else{
                targetCode = emrFaultDetail.getTargetCode() + "";
            }
            //定义退回内容
            String remark = emrFaultDetail.getBackContent();
            List<Integer> backToCodeList = new ArrayList<>();
            //参数输入
            SimpleDateFormat format1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            String nowTime = format1.format(new Date());
            //修改复审内容
            emrFaultDetail.setCreater(username);
            emrFaultDetail.setCreateTime(nowTime);
            //定义修改支线
            String roleIndex = "";
            switch (returntoRole) {
                //退回给医生
                case "DOCTOR":
                    //获取当前状态码
                    status = EnumVerify.DocState.backToDoctor(Integer.valueOf(emrFaultDetail.getStatus()),archiveFlowRoles);
                    //退回的状态码
                    backToCodeList.add(EnumVerify.DocState.DortorBack.getCode());
                    roleIndex = "1";
                    break;
                //退回给护士
                case "NURSE":
                    status = EnumVerify.DocState.backToNurse(Integer.valueOf(emrFaultDetail.getStatus()),archiveFlowRoles);
                    backToCodeList.add(EnumVerify.DocState.NurseBack.getCode());
                    roleIndex = "2";
                    break;
                //退回给医生和护士
                case "ALL":
                    status = EnumVerify.DocState.backToAll();
                    backToCodeList.add(EnumVerify.DocState.DortorBack.getCode());
                    backToCodeList.add(EnumVerify.DocState.NurseBack.getCode());
                    break;
                default:;
            }
            //循环添加进流转记录和缺陷记录
            for(Integer backToCode : backToCodeList){
                //获取当前节点和下个节点对象集合
                List<ArchiveFlowRole> currentRoleAndNextRole = flowInfoService.getCurrentRoleAndNextRole(targetCode, backToCode,archiveFlowRoles);
                //添加进流转记录
                ArchiveFlowInfo flowInfo = flowInfoService.insertFlowInfo(masterId,remark,currentRoleAndNextRole.get(0),currentRoleAndNextRole.get(1),new Date());
                //插入退回信息
                //backInsertOracle(0,flowInfo);
                //修改退回信息
//
//                updateOracle(roleIndex,emrFaultDetail.getArchiveDetailId());
                emrFaultDetail.setParentId(flowInfo.getId());
                //添加缺陷记录
                emrFaultDetailService.insertSel(emrFaultDetail);
            }
            archiveMaster.setArchivestate(status + "");
        }
        archiveMaster.setId(emrFaultDetail.getArchiveDetailId());
        archiveMaster.setReturntoRole(returntoRole);
        //设置回退操作人信息
        archiveMaster.setReturnOperUsername(username);
        updateByClo(archiveMaster);

        //退回金蝶
        //获取基本信息
        /*Archive_Master archiveMaster1 = selectByPrimaryKey(emrFaultDetail.getArchiveDetailId());
        try {
            String msg = "<req>" +
                    "<patNo>" + archiveMaster1.getPatientId() + "</patNo>" +
                    "<patTimes>" + archiveMaster1.getVisitId() + "</patTimes>" +
                    "<patFileNo>" + archiveMaster1.getInpNo() + "</patFileNo>" +
                    "<returnObject>" + emrFaultDetail.getReturntoRole() + "</returnObject>" +
                    "<returnType>01</returnType>" +
                    "<returnName>修改病历</returnName>" +
                    "<returnDept>" + archiveMaster1.getDeptName() + "</returnDept>" +
                    "<regEmp>" + username + "</regEmp>" +
                    "<regDept>8913</regDept>" +
                    "<returnDate>" + nowTime + "</returnDate>" +
                    "<validHours>24</validHours>" +
                    "</req>";
            log.info(msg);
            JAXDynamicClientFactory clientFactory = JAXDynamicClientFactory.newInstance();
            String url = "http://192.168.10.39:8001/KingdeeEmrArchiveReturn.asmx?wsdl";
            String param1 = "http://tempuri.org/";
            String param2 = "SetArchieveReturn";
            Client client = clientFactory.createClient(url);
            QName qname = new QName(param1, param2);
            Object[] result = client.invoke(qname, msg);
            *//*Document document = DocumentHelper.parseText(result[0].toString());
            Element rootElement = document.getRootElement();
            Iterator iterator = rootElement.elementIterator();
            while (iterator.hasNext()) {
                Element element = (Element) iterator.next();
                if ("resultCode".equals(element.getName())) {
                    System.out.println("resultCode:" + element.getTextTrim());
                }
                if ("resultDesc".equals(element.getName())) {
                    System.out.println("resultDesc:" + element.getTextTrim());
                }
            }*//*
            log.info("回退成功!");
        }catch(Exception e){
            log.info("回退出错了!");
            log.info(e.getMessage());
            throw new  RuntimeException();
        }*/
        return 1;
    }

    @Override
    public List<Archive_Master> selectByObject(Archive_Master master) {
        return archiveMasterMapper.selectByObject(master);
    }

    /**
     * author:zwh
     * 初审审核操作
     * */
    @Override
    public Msg updateStateByArchivId(Archive_Master_Vo master) throws Exception{
        //修改病案归档状态
        if (StringUtils.isNotBlank(master.getArchivestate())) {
            StringBuilder masterIds = new StringBuilder();
            //查询审核角色集合，根据有效进行计算状态code
            List<ArchiveFlowRole> archiveFlowRoles = flowRoleMapper.selectAll();
            //获取目前code集合
            String[] archivestates = master.getArchivestate().split(",");
            //获取主键id集合
            String[] ids = master.getId().split(",");
            //获取审核角色集合
            String[] roleArr = master.getRoles().split(",");
            //获取当前操作code集合
            String[] roleCodes = master.getStatus().split(",");
            //获取资料完整标识集合
            String[] isSuccesses = master.getIsSuccess().split(",");
            for (int i = 0; i < archivestates.length; i++) {
                if(StringUtils.isNotBlank(archivestates[i])){
                    //完整或只有一个
                    if("1".equals(isSuccesses[i]) || ids.length == 1){
                        updateApprove(ids[i],master.getRemark(), roleArr[i], archiveFlowRoles, archivestates[i], roleCodes[i]);
                    }else{
                        if(StringUtils.isBlank(masterIds)){
                            masterIds.append("'").append(ids[i]).append("'");
                        }else{
                            masterIds.append(",'").append(ids[i]).append("'");
                        }
                    }
                }
            }
            //不完整,根据不完整主键集合组织不对应不完整分类集合
            List<Archive_Master_Vo> unSuccessList = new ArrayList<>();
            if(StringUtils.isNotBlank(masterIds)){
                unSuccessList = selectUnSuccessAssortByMasterIds(masterIds);
            }
            if(!CollectionUtils.isEmpty(unSuccessList)){
                //存在不完整
                return Msg.success().add("unSuccessList",unSuccessList);
            }else{
                //全部完整返回
                return Msg.success();
            }
        } else {
            return Msg.fail("初始状态出错了!");
        }
    }

    /**
     * 插入第三方oracle库
     * @param flowInfo
     */
    private void insertOracle(Integer code,ArchiveFlowInfo flowInfo){
        //查询是否存在记录
        String masterId = flowInfo.getMasterId();
        String selectSql = "SELECT count(1) FROM ARCHIVE_FLOW_INFO where master_id = '"+masterId+"'";
        List<String> counts = null;
        try {
            counts = OracleConnect.selectList(selectSql);
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException("审核时查询oracle记录失败");
        }
        log.info("masterId查询结果:"+counts.get(0));
        if(code == 1 || !"0".equals(counts.get(0))) {
            Archive_Master master = archiveMasterMapper.selectByPrimaryKey(masterId);
            String patientId = master.getPatientId();
            String sql = "insert all into archive_flow_info (" +
                    "master_id, " +
                    "step_index," +
                    "start_step_id, " +
                    "start_step, " +
                    "sumbit_name," +
                    "target_step_id, " +
                    "target_step, " +
                    "oper_role," +
                    "use_seconds, " +
                    "remark, " +
                    "user_name," +
                    "sumbit_time, " +
                    "createtime," +
                    "jzh," +
                    "is_available" +
                    ")" +
                    "values ("
                    + "'" + flowInfo.getMasterId() + "',"
                    + flowInfo.getStepIndex() + ","
                    + flowInfo.getStartStepId() + ","
                    + "'" + flowInfo.getStartStep() + "',"
                    + "'" + flowInfo.getSumbitName() + "',"
                    + flowInfo.getTargetStepId() + ","
                    + "'" + flowInfo.getTargetStep() + "',"
                    + flowInfo.getOperRole() + ","
                    + "'" + flowInfo.getUseSeconds() + "',"
                    + "'" + flowInfo.getRemark() + "',"
                    + "'" + flowInfo.getUserName() + "',"
                    + "SYSDATE,"
                    + "SYSDATE,"
                    + "'" + patientId + "',"
                    + "'" + "是" + "'"
                    + ") select * from dual";
            log.info("sql:"+sql);
            try {
                OracleConnect.insert(sql);
                log.info("成功");
            } catch (Exception e) {
                e.printStackTrace();
                log.info("失败");
                throw new RuntimeException("审核信息插入oracle失败");
            }

        }
    }

    /**
     * 当次初审
     * @param remark
     * @param role
     * @param archiveFlowRoles
     * @param archivestate
     * @param masterId
     * @param roleCode
     * @throws Exception
     */
    private void updateApprove(String masterId,String remark, String role, List<ArchiveFlowRole> archiveFlowRoles, String archivestate, String roleCode) throws Exception {
        Archive_Master archiveMaster = new Archive_Master();
        Integer code = Integer.valueOf(archivestate);
        //计算状态code
        // code:当前状态code，role:角色1医生2护士3护士长，archiveFlowRoles：根据有效的审核角色进行计算
        int statuCode = EnumVerify.DocState.GetNextStepCode(code,Integer.valueOf(role), archiveFlowRoles);
        archiveMaster.setArchivestate(statuCode+"");
        archiveMaster.setId(masterId);
        //更新
        updateByClo(archiveMaster);
        //获取当前节点和下个节点对象集合
        List<ArchiveFlowRole> currentRoleAndNextRole = flowInfoService.getCurrentRoleAndNextRole(roleCode, null,archiveFlowRoles);
        //添加进流转记录
        ArchiveFlowInfo flowInfo = flowInfoService.insertFlowInfo(masterId,remark,currentRoleAndNextRole.get(0),currentRoleAndNextRole.get(1),new Date());
        //处于初始状态的添加进第三方oracle库里
        //insertOracle(code,flowInfo);
    }

    /**
     * 根据不完整masterId集合返回对应不完整分类集合
     * @param masterIds
     * @return
     */
    private List<Archive_Master_Vo> selectUnSuccessAssortByMasterIds(StringBuilder masterIds) {
        List<Archive_Master_Vo> list = new ArrayList<>();
        if(StringUtils.isNotBlank(masterIds)){
            //按masterId分组查询需要已采集的分类集合
            List<Archive_Detail> details = detailMapper.selectAssortIdsByMasterIds(masterIds.toString());
            //按masterId分组查询是否存在缺少检验检查报告单的masterId集合
            List<Archive_Detail> inspectionList = detailMapper.selectNotInspectionList(masterIds.toString());
            //查询需要校验完整性的分类集合
            Zd_Assort assort = new Zd_Assort();
            assort.setIsCheck(1);
            List<Zd_Assort> assortList = assortMapper.selectAll(assort);
            String[] masterIdArr = masterIds.toString().split(",");
            for (String masterId:masterIdArr) {
                masterId = masterId.replaceAll("\'", "");
                Archive_Master_Vo master = new Archive_Master_Vo();
                StringBuilder unSuccessAssortNames = new StringBuilder();
                for(Zd_Assort assortFor : assortList){
                    //定义无匹配的分类
                    boolean flag = false;
                    for(Archive_Detail detail : details){
                        //判断主键相同且分类id相同
                        if(masterId.equals(detail.getMasterid()) && assortFor.getAssortId().equals(detail.getAssortid())){
                            flag = true;
                            break;
                        }
                    }
                    //该分类无匹配组织名称集合
                    if(!flag){
                        if(StringUtils.isBlank(unSuccessAssortNames)){
                            unSuccessAssortNames.append(assortFor.getAssortName());
                        }else{
                            unSuccessAssortNames.append(",").append(assortFor.getAssortName());
                        }
                    }
                }
                //判断是否缺少检验检查报告单
                if(!CollectionUtils.isEmpty(inspectionList)){
                    //定义存在缺少检验检查报告
                    boolean inspectionFlag = true;
                    for(Archive_Detail detail:inspectionList){
                        if(masterId.equals(detail.getMasterid())){
                            inspectionFlag = false;
                            break;
                        }
                    }
                    if(!inspectionFlag){
                        if(StringUtils.isNotBlank(unSuccessAssortNames)){
                            unSuccessAssortNames.append(",");
                        }
                        unSuccessAssortNames.append("检验检查报告单");
                    }
                }
                master.setId(masterId);
                master.setUnSuccessAssortNames(unSuccessAssortNames.toString());
                list.add(master);
            }
        }
        return list;
    }

    /**
     * author:zwh
     * 终审审核操作
     * */
    @Override
    public void updateFiled(Archive_Master_Vo master) throws Exception{
        // 从session获取用户名
        String username = (String) SecurityUtils.getSubject().getSession().getAttribute("userSession");//获取前面登录的用户名
        SimpleDateFormat format1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String ids = master.getId();
        String[] idList = ids.split(",");
        List<ArchiveFlowRole> archiveFlowRoles = flowRoleMapper.selectAll();
        List<Archive_Master> archive_masterVos = archive_masterMapper.selectCommonlyUsed(master);
        if(CollectionUtils.isEmpty(archive_masterVos)){
            return;
        }
        Archive_Master archive_master_vos = archive_masterVos.get(0);
        CommomVo commomVo=new CommomVo();
            commomVo.setPatientId(archive_master_vos.getId());
            commomVo.setAdmissTimes(archive_master_vos.getVisitId());
            commomVo.setAdmissId(archive_master_vos.getInpNo());
            commomVo.setInpatientNo(archive_master_vos.getInpNo());
            commomVo.setName(archive_master_vos.getName());
            commomVo.setCheckedDatetime(format1.format(new Date()));
            commomVo.setSex(archive_master_vos.getSex());
            commomVo.setAdmissId(archive_master_vos.getId());
            String substring = archive_master_vos.getSubAssort().substring(0, 2);
            commomVo.setAge(substring);
        DateFormat format = new SimpleDateFormat("yyyy-MM-dd");
        Date date1=null;
        Date date2=null;
        String dischargeDateTime = archive_master_vos.getDischargeDateTime();
        String admissionDateTime = archive_master_vos.getAdmissionDateTime();
        try {
            date1=format.parse(dischargeDateTime);
            date2=format.parse(admissionDateTime);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        Calendar ca1 = Calendar.getInstance();
        Calendar ca2 = Calendar.getInstance();
        ca1.setTime(date1);
        ca2.setTime(date2);
        long distanceMin =( ca1.getTimeInMillis()- ca2.getTimeInMillis())/(1000*60*60*24);
        int i2= (int)distanceMin;
        commomVo.setAdmissDays(i2);
        int i = Integer.parseInt(substring);
        int i1 = i * 12;
        String s = String.valueOf(i1);
        commomVo.setAgeMonth(s);
            commomVo.setAdmissDate(archive_master_vos.getAdmissionDateTime());
            commomVo.setDisDate(archive_master_vos.getDischargeDateTime());
            commomVo.setDisDept(archive_master_vos.getDeptName1());
            commomVo.setAttending(archive_master_vos.getDoctorInCharge());
             archive_masterMapper.insertCommonlyUsed(commomVo);
        for(String id : idList) {
            if(StringUtils.isNotBlank(id)) {
                //修改病案归档状态
                Archive_Master archiveMaster = new Archive_Master();
                master.setId(id);
                //拷贝对象
                BeanUtils.copyProperties(master, archiveMaster);
                //定义提交时间
                //设置审核时间
                Date sumbitDate = new Date();
                if (StringUtils.isBlank(master.getCheckedDatetime())) {
                    //未手动修改终审时间，使用当前时间
                    archiveMaster.setCheckedDatetime(format1.format(new Date()));
                } else {
                    //手动修改终审时间，是否修改的时间
                    archiveMaster.setCheckedDatetime(master.getCheckedDatetime());
                    sumbitDate = format1.parse(master.getCheckedDatetime());
                }
                //设置审核人
                archiveMaster.setCheckedDoctor(username);
                //设为已归档状态
                archiveMaster.setArchivestate(EnumVerify.DocState.MedicalRoomCheck.getCode() + "");
                //更新
                updateByClo(archiveMaster);
                //当前节点两种可能病案室审核和已召回
                //是已召回，将当前节点code置为病案室审核
                String currentStatus = EnumVerify.DocState.MedicalRoomCheck.getCode() + "";
                //获取当前节点和下个节点对象集合
                List<ArchiveFlowRole> currentRoleAndNextRole = flowInfoService.getCurrentRoleAndNextRole(currentStatus, EnumVerify.DocState.MedicalRoomCheck.getCode(),archiveFlowRoles);
                //添加流转节点
                ArchiveFlowInfo flowInfo = flowInfoService.insertFlowInfo(master.getId(), master.getRemark(), currentRoleAndNextRole.get(0), currentRoleAndNextRole.get(1), sumbitDate);
                //insertOracle(0,flowInfo);
                //添加纸质位置
                paperPostionService.savePaperPostion(master);
            }
        }
    }

    /**
     * 获取当前角色属于审核哪种角色
     * 医生支线返回1，护士支线返回2，一般审核者返回3
     * */
    @Override
    public Msg getRole(){
        Msg msg = new Msg();
        //获取用户功能权限信息
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        Power_User user = (Power_User) request.getSession().getAttribute("CURRENT_USER");
        Set<String> menus = user.getMenus();
        if(null != menus && !menus.isEmpty()){
            //定义医生支线role
            int doctorRole = 0;
            //定义护士支线role
            int nurseRole = 0;
            //护士长审核权限标识
            String headNurseCheck = "/beHosp/headNurseCheck";
            //医生提交权限标识
            String doctorSumbit = "/beHosp/doctorSumbit";
            //医生质控员质控权限标识
            String doctorCheckSumbit = "/beHosp/doctorCheckSumbit";
            //科主任审核权限标识
            String directorCheckSumbit = "/beHosp/directorCheckSumbit";
            //护士提交权限标识
            String nurseSumbit = "/beHosp/nurseSumbit";
            //护士质控员质控权限标识
            String nurseCheckSumbit = "/beHosp/nurseCheckSumbit";
            int genegrateRole = 0;
            //拼接有权限的code
            StringBuilder powerCode = new StringBuilder();
            //遍历，判断拥有权限，拥有护士长权限立即返回4，拥有医生支线赋值doctorRole=1,拥有护士支线赋值nurseRole=1
            for (String menu : menus) {
                if(StringUtils.isNotBlank(menu)){
                    if(menu.equals(doctorSumbit)){
                        doctorRole = 1;
                        powerCode.append(EnumVerify.DocState.DortorSubmit.getCode()).append(",");
                    }
                    if(menu.equals(doctorCheckSumbit)){
                        doctorRole = 1;
                        powerCode.append(EnumVerify.DocState.DortorCheck.getCode()).append(",");
                    }
                    if(menu.equals(directorCheckSumbit)){
                        doctorRole = 1;
                        powerCode.append(EnumVerify.DocState.DirectorCheck.getCode()).append(",");
                    }
                    if(menu.equals(nurseSumbit)){
                        nurseRole = 1;
                        powerCode.append(EnumVerify.DocState.NurseSubmit.getCode()).append(",");
                    }
                    if(menu.equals(nurseCheckSumbit)){
                        nurseRole = 1;
                        powerCode.append(EnumVerify.DocState.NurseCheck.getCode()).append(",");
                    }
                    if(menu.equals(headNurseCheck)){
                        genegrateRole = 1;
                        powerCode.append(EnumVerify.DocState.HeadNurseCheck.getCode()).append(",");
                    }
                }
            }
            if(powerCode.toString().endsWith(",")){
                powerCode = new StringBuilder(powerCode.substring(0, powerCode.length() - 1));
            }
            msg.setMsg(powerCode.toString());
            //判断属于哪个支线权限
            if(doctorRole == 1 && nurseRole == 0){
                msg.setCode(1);
            }
            if(doctorRole == 0 && nurseRole == 1){
                msg.setCode(2);
            }
            if(doctorRole == 1 && nurseRole == 1 || genegrateRole == 1){
                msg.setCode(4);
            }
            return msg;
        }
        return null;
    }

    /**
     * 查oracle视图，带记账号查询纸质签收的记账号集合
     * @param patientIdStr
     * @return
     */
    @Override
    public List<String> selectOracleGetSignList(StringBuilder patientIdStr, List<String> selectList){
        //查询签收情况
        String sql = "";
        if(StringUtils.isNotBlank(patientIdStr)){
            sql = "select fpat_no from v_jswzh_archive_bqreg where fpat_no in ("+patientIdStr.toString()+")" +
                "and  freg_date is not null";
        }else{
            sql = "select fpat_no from v_jswzh_archive_bqreg where freg_date is not null";
        }
        //执行查询
        try{
            selectList = OracleConnect.selectList(sql);
        }catch (Exception e){
            e.printStackTrace();
            ExceptionPrintUtil.printException(e);
        }
        return selectList;
    }

    /*@Override
    public String selectLockInfoByMasterId(String archiveDetailId) {
        return null;
    }*/
    @Override
    public String selectAssortIdsByMasterId(String masterId) {
        //按masterId分组查询需要已采集的分类集合
        List<Archive_Detail> details = detailMapper.selectAssortIdsByMasterId(masterId.toString());
        String assortIdStr ="";
        for (Archive_Detail detail : details) {
            if(StringUtils.isNotBlank(detail.getAssortid())){
                assortIdStr+=detail.getAssortid()+",";
            }
        }
        return assortIdStr;
    }
}


