package com.emr.service.ipml;

import com.alibaba.fastjson.JSON;
import com.emr.dao.CommomMapper;
import com.emr.entity.Power_User;
import com.emr.vo.User;
import net.sf.json.JSONObject;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.apache.shiro.util.CollectionUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;

/**
 * @ProjectName:
 * @Description:
 * @Param 传输参数
 * @Return
 * @Author: 曾文和
 * @CreateDate: 2020/10/29 13:21
 * @UpdateUser: 曾文和
 * @UpdateDate: 2020/10/29 13:21
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
@Service
@Transactional
public class CommomService {
    @Value("${POWER_URLHEAD}")
    private String POWER_URLHEAD;
    /**
     * 批量添加
     * @param tableName 表名
     * @param list 批量添加的集合数据
     * @param commomMapper 调用执行mapper接口
     */
    public void insertBatch(String tableName, List list,CommomMapper commomMapper){
        if(!CollectionUtils.isEmpty(list)){
            //根据list数量插入
            ///查询表列数
            int colCount = commomMapper.selectColByTableName(tableName);
            int simpleInsertCount = 2100/colCount;
            List<Object> insertList = new ArrayList<>();
            for (int i = 0; i < list.size(); i++) {
                //处理批量添加sql太长问题，设定个数为180
                if(i != 0 && insertList.size()%simpleInsertCount == 0){
                    insertList.clear();
                }
                insertList.add(list.get(i));
                //是180的倍数或最后一个
                if(insertList.size()%simpleInsertCount == 0 || i == list.size() - 1){
                    //批量添加
                    commomMapper.insertBatch(insertList);
                }
            }
        }
    }

    /**
     * 适用可直接从session中取用户列表
     * @return
     * @throws Exception
     */
    public  List<User> getUserList() throws Exception{
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        List<User> userList = (List<User>)request.getSession().getAttribute("USER_LIST");
        if(null == userList || userList.isEmpty()){
            //调用接口查询
            String resultString = "";
            // 创建uri
            String url = POWER_URLHEAD+"/font/getUserList?userName=admin";
            // 执行请求
            CloseableHttpResponse response = HttpClients.createDefault().execute(new HttpGet(url));
            // 判断返回状态是否为200
            if (response.getStatusLine().getStatusCode() == 200) {
                resultString = EntityUtils.toString(response.getEntity(), "UTF-8");
            }
            JSONObject jsonObject = JSONObject.fromObject(resultString);
            String code = jsonObject.getString("code");
            if("500".equals(code)) {
                throw new RuntimeException(resultString);
            }
            if("100".equals(code)) {
                String extend = jsonObject.getString("extend");
                JSONObject extendObject = JSONObject.fromObject(extend);
                String userList1 = extendObject.getString("userList");
                userList = JSON.parseArray(userList1, User.class);
                //设置进session
                request.getSession().setAttribute("USER_LIST", userList);
            }
        }
        return userList;
    }

    /**
     * 适用可不能从session中取用户列表，直接调用权限系统接口
     * @return
     * @throws Exception
     */
    public  List<User> getUserList(String userName) throws Exception{
        //调用接口查询
        String resultString = "";
        // 创建uri
        String url = POWER_URLHEAD+"/font/getUserList?userName="+userName;
        // 执行请求
        CloseableHttpResponse response = HttpClients.createDefault().execute(new HttpGet(url));
        // 判断返回状态是否为200
        if (response.getStatusLine().getStatusCode() == 200) {
            resultString = EntityUtils.toString(response.getEntity(), "UTF-8");
        }
        JSONObject jsonObject = JSONObject.fromObject(resultString);
        String extend = jsonObject.getString("extend");
        JSONObject extendObject = JSONObject.fromObject(extend);
        String userList1 = extendObject.getString("userList");
        List<User> userList = JSON.parseArray(userList1,User.class);
        //设置进session
        return userList;
    }

    /**
     * 根据用户获取session中用户集合的中对应的科室编号deptCode
     * @param userName
     * @return
     * @throws Exception
     */
    public String selectDeptCodeByUser(String userName) throws Exception {
        List<User> userList = getUserList();
        if(!CollectionUtils.isEmpty(userList)){
            for(User user : userList){
                if(user.getUserName().equals(userName)){
                    return user.getDeptCode();
                }
            }
        }
        return null;
    }

    /**
     * 获取登录用户对象
     * @param request
     * @return
     */
    public Power_User getCurrentUser(HttpServletRequest request){
        Power_User user = (Power_User)request.getSession().getAttribute("CURRENT_USER");
        return user;
    }

    /**
     * 获取登录用户名
     * @param request
     * @return
     */
    public String getCurrentUserName(HttpServletRequest request){
        Power_User currentUser = getCurrentUser(request);
        if(null != currentUser){
            return currentUser.getUserName();
        }else{
            return null;
        }
    }
}
