package com.emr.service.ipml;

import com.alibaba.fastjson.JSON;
import com.emr.dao.*;
import com.emr.entity.*;
import com.emr.quart.DynamicScheduleTaskSecond;
import com.emr.service.Emr_DictionaryService;
import com.emr.vo.CommomTree;
import com.emr.vo.User;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.util.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @ProjectName:
 * @Description:
 * @Param 传输参数
 * @Return
 * @Author: 曾文和
 * @CreateDate: 2020/10/27 10:55
 * @UpdateUser: 曾文和
 * @UpdateDate: 2020/10/27 10:55
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
@Service
@Transactional
public class EmrOvertimeSetService {
    @Value("${overDateSet}")
    private String overDateSet;
    @Autowired
    private EmrOvertimeSetMapper overtimeSetMapper;
    @Autowired
    private Emr_DictionaryService dictionaryService;
    @Autowired
    private ArchiveRemindcancleDeptMapper remindcancleDeptMapper;
    @Autowired
    private ArchiveRemindDeptUserMapper remindDeptUserMapper;
    @Autowired
    private CommomService commomService;
    @Autowired
    private DynamicScheduleTaskSecond dynamicScheduleTaskSecond;
    @Autowired
    private EmrHolidaySetMapper holidaySetMapper;
    /**
     * 根据主键查询
     * @return
     */
    public EmrOvertimeSet selectByPrimaryKey(Integer id){
        return overtimeSetMapper.selectByPrimaryKey(id);
    }

    /**
     * 更新超期基础设置表
     * @param overtimeSet
     */
    public void updateOverTimeSet(EmrOvertimeSet overtimeSet){
        overtimeSetMapper.updateByPrimaryKeySelective(overtimeSet);
        //重新执行任务
        dynamicScheduleTaskSecond.setCron(overtimeSet.getRemindTime());
    }

    /**
     * 查询取消提醒科室树
     * @return
     */
    public List<CommomTree> selectRemindDeptTree() throws Exception {
        List<CommomTree> list = new ArrayList<>();
        //查询科室
        List<Emr_Dictionary> dictionaries = dictionaryService.dicByClo(new Emr_Dictionary());
        if(!CollectionUtils.isEmpty(dictionaries)){
            //查询取消科室
            List<ArchiveRemindcancleDept> archiveRemindcancleDepts = remindcancleDeptMapper.selectAll();
            for(Emr_Dictionary dictionary : dictionaries) {
                //获取编码
                String code = dictionary.getCode();
                if(StringUtils.isNotBlank(code)) {
                    CommomTree tree1 = new CommomTree();
                    tree1.setId(dictionary.getId());
                    tree1.setCode(dictionary.getCode());
                    tree1.setName(dictionary.getName());
                    tree1.setParentId(0);
                    //判断是否有对应取消提醒的科室选中
                    if(!CollectionUtils.isEmpty(archiveRemindcancleDepts)){
                        for(ArchiveRemindcancleDept obj : archiveRemindcancleDepts){
                            if(obj.getDeptCode().equals(code)){
                                tree1.setChecked("true");
                                break;
                            }
                        }
                    }
                    list.add(tree1);
                }
            }
        }
        return list;
    }
    /**
     * 查询取消科室提醒单独用户树
     * @return
     */
    public List<CommomTree> selectRemindDeptUserTree() throws Exception {
        //查询科室
        List<Emr_Dictionary> dictionaries = dictionaryService.dicByClo(new Emr_Dictionary());
        List<CommomTree> list = new ArrayList<>();
        if(!CollectionUtils.isEmpty(dictionaries)){
            //查询用户信息
            List<User> userList = commomService.getUserList();
            if(!CollectionUtils.isEmpty(userList)){
                //查询取消科室提醒单独提醒的用户
                List<ArchiveRemindDeptUser> remindDeptUsers = remindDeptUserMapper.selectAll();
                //定义角色id集合
                String roleListJson = "[" +
                        "{'roleId':1,roleName:'医生'}," +
                        "{'roleId':5,roleName:'医生质控员'}," +
                        "{'roleId':6,roleName:'科主任'}," +
                        "{'roleId':2,roleName:'护士'}," +
                        "{'roleId':8,roleName:'护士质控员'}," +
                        "{'roleId':7,roleName:'护士长'}," +
                        "{'roleId':3,roleName:'病案室人员'}" +
                        "]";
                List<User> roleList = JSON.parseArray(roleListJson, User.class);
                //设定用户id从20000开始
                int id = 1;
                int twoParentId = 0;
                int threeParentId = 0;
                //组织用户
                for(Emr_Dictionary dictionary : dictionaries) {
                    //获取科室编码
                    String code = dictionary.getCode();
                    if(StringUtils.isNotBlank(code)) {
                        //第一层，科室层
                        CommomTree tree1 = new CommomTree();
                        tree1.setId(id);
                        twoParentId = id;
                        id++;
                        tree1.setCode(dictionary.getCode());
                        tree1.setName(dictionary.getName());
                        tree1.setParentId(0);
                        //过滤字典类别科室 科室code=null
                        //定义存在用户的科室
                        boolean deptFlag = false;
                        boolean deptCheckedFlag = false;
                        for(User role : roleList){
                            //第二层，审核角色层
                            CommomTree tree2 = new CommomTree();
                            tree2.setId(id);
                            threeParentId = id;
                            id++;
                            tree2.setParentId(twoParentId);
                            tree2.setName(role.getRoleName());
                            //定义存在用户的角色
                            boolean roleFlag = false;
                            boolean roleCheckedFlag = false;
                            for (User user : userList) {
                                //第三层，用户层
                                //获取用户所属科室
                                String deptCode = user.getDeptCode();
                                if (StringUtils.isNotBlank(deptCode) && deptCode.equals(code) && role.getRoleId().equals(user.getRoleId())) {
                                    CommomTree tree3 = new CommomTree();
                                    tree3.setId(id);
                                    id++;
                                    tree3.setParentId(threeParentId);
                                    String name = user.getName();
                                    if(StringUtils.isNotBlank(name)){
                                        tree3.setName(user.getUserName() + "-" + name);
                                    }else{
                                        tree3.setName(user.getUserName());
                                    }
                                    //设置工号
                                    tree3.setRealName(user.getUserName());
                                    //设置科室编号
                                    tree3.setCode(deptCode);
                                    //设置角色id
                                    tree3.setRealParentId(user.getRoleId());
                                    //判断是否用户选中
                                    if(!CollectionUtils.isEmpty(remindDeptUsers)){
                                        for(ArchiveRemindDeptUser remindDeptUser : remindDeptUsers){
                                            if(remindDeptUser.getUserName().equals(user.getUserName())){
                                                tree3.setChecked("true");
                                                deptCheckedFlag = true;
                                                roleCheckedFlag = true;
                                                break;
                                            }
                                        }
                                    }
                                    list.add(tree3);
                                    deptFlag = true;
                                    roleFlag = true;
                                }
                            }
                            //角色不存在用户则移除。不添加
                            if(roleFlag){
                                if(roleCheckedFlag) {
                                    tree2.setChecked("true");
                                }
                                list.add(tree2);
                            }
                        }
                        //科室不存在用户则移除。不添加
                        if(deptFlag){
                            if(deptCheckedFlag) {
                                tree1.setChecked("true");
                            }
                            list.add(tree1);
                        }
                    }
                }
            }
        }
        return list;
    }

    /**
     * 保存取消科室树
     * @param deptIds
     */
    public void updateRemindCancleSet(String deptIds) {
        remindcancleDeptMapper.updateRemindCancleSet(deptIds);
    }

    /**
     * 保存取消提醒科室单独提醒个别用户集合
     * @param list
     */
    public void updateRemindDeptUserSet(List<ArchiveRemindDeptUser> list){
        //先清空表
        remindDeptUserMapper.deleteRemindDeptUser();
        //批量插入
        commomService.insertBatch("archive_remind_dept_user",list,remindDeptUserMapper);
    }

    /**
     * 获取即将超期通知的日期
     * @return
     */
    public String getRemindDate(){
        SimpleDateFormat fmt = new SimpleDateFormat("yyyy-MM-dd");
        //获取工作日集合
        List<EmrHolidaySet> workDateList = holidaySetMapper.selectWorkDayByDate(overDateSet, fmt.format(new Date()), "date desc");
        if(!CollectionUtils.isEmpty(workDateList)){
            //查询超期设置参数
            EmrOvertimeSet emrOvertimeSet = overtimeSetMapper.selectByPrimaryKey(1);
            //获取提前超期天数
            Integer surplusDays = emrOvertimeSet.getSurplusDays();
            if(workDateList.size() >= surplusDays){
                return workDateList.get(emrOvertimeSet.getDays() - surplusDays + 1).getDate();
            }
        }
        return null;
    }
}
