package com.emr.service.ipml;

import com.emr.dao.EmrPaperPositionDictionaryMapper;
import com.emr.dao.EmrPaperPositionInfoMapper;
import com.emr.entity.Archive_Master_Vo;
import com.emr.entity.EmrPaperPositionDictionary;
import com.emr.entity.EmrPaperPositionInfo;
import com.emr.util.ExceptionPrintUtil;
import com.emr.util.IDHelper;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.util.CollectionUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Field;
import java.text.SimpleDateFormat;
import java.util.*;

@Service
@Transactional
public class PaperPostionService {
    @Autowired
    private EmrPaperPositionDictionaryMapper paperPositionDictionaryMapper;
    @Autowired
    private EmrPaperPositionInfoMapper paperPositionInfoMapper;
    @Autowired
    private CommomService commomService;

    public List<EmrPaperPositionDictionary> selectPaperPositonDictionary(){
        //查询全部
        List<EmrPaperPositionDictionary> list = paperPositionDictionaryMapper.selectAll();
        //定义返回列表
        List<EmrPaperPositionDictionary> returnList = new ArrayList<>();
        if(!CollectionUtils.isEmpty(list)){
            //定义柜子列名集合
            Set<String> set = new LinkedHashSet<>();
            for(EmrPaperPositionDictionary obj : list){
                set.add(obj.getTypename());
            }
            for(String typeName : set){
                EmrPaperPositionDictionary vo = new EmrPaperPositionDictionary();
                vo.setTypename(typeName);
                //定义接收柜子列值集合
                StringBuilder typeValues = new StringBuilder();
                for(EmrPaperPositionDictionary obj : list){
                    if(obj.getTypename().equals(typeName)){
                        if(StringUtils.isNotBlank(typeValues)){
                            typeValues.append(",");
                        }
                        typeValues.append(obj.getTypevalue());
                        vo.setInt1(obj.getInt1());
                        vo.setInt2(obj.getInt2());
                    }
                }
                vo.setTypevalue(typeValues.toString());
                returnList.add(vo);
            }
        }
        return returnList;
    }

    public void savePaperPostion(Archive_Master_Vo archiveMaster) {
        if(StringUtils.isNotBlank(archiveMaster.getPaperPosition()) || StringUtils.isNotBlank(archiveMaster.getPaperPosition1()) || StringUtils.isNotBlank(archiveMaster.getPaperPosition2()) || StringUtils.isNotBlank(archiveMaster.getPaperPosition3()) || StringUtils.isNotBlank(archiveMaster.getPaperPosition4())) {
            EmrPaperPositionInfo obj = new EmrPaperPositionInfo();
            BeanUtils.copyProperties(archiveMaster, obj);
            updatePaperPostion(obj);
        }
    }

    /**
     * 处理纸质信息值带横杠的拆横杠，多个值的用横杠拼接
     * @param obj
     */
    private void handlePaperPostion(EmrPaperPositionInfo obj) {
        if(StringUtils.isNotBlank(obj.getPaperPosition())){
            //带横杠的拆横杠
            String[] paperPositions = obj.getPaperPosition().split("-");
            for(int i = 0;i < paperPositions.length;i++){
                if(StringUtils.isNotBlank(paperPositions[i])) {
                    //组织字段值
                    String fieldName = "paperPosition" + (i + 1);
                    try{
                        Field field = obj.getClass().getDeclaredField(fieldName);
                        field.setAccessible(true);
                        field.set(obj,paperPositions[i]);
                    }catch (Exception e){
                        e.printStackTrace();
                        ExceptionPrintUtil.printException(e);
                    }
                }
            }
        }else{
            //多个值的用横杠拼接
            String paperPosition = obj.getPaperPosition1();
            paperPosition += "-";
            if(StringUtils.isNotBlank(obj.getPaperPosition2())){
                paperPosition += obj.getPaperPosition2();
            }
            paperPosition += "-";
            if(StringUtils.isNotBlank(obj.getPaperPosition3())){
                paperPosition += obj.getPaperPosition3();
            }
            paperPosition += "-";
            if(StringUtils.isNotBlank(obj.getPaperPosition4())){
                paperPosition += obj.getPaperPosition4();
            }
            obj.setPaperPosition(paperPosition);
        }
    }

    /**
     * 修改纸质表
     * @param obj
     */
    public void updateByPrimaryKeySelectiveUpdatePaperPosition(EmrPaperPositionInfo obj){
        SimpleDateFormat fmt = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String time = fmt.format(new Date());
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String currentUserName = commomService.getCurrentUserName(request);
        obj.setUpdateUserName(currentUserName);
        obj.setUpdateTime(time);
        paperPositionInfoMapper.updateByPrimaryKeySelective(obj);
    }

    /**
     * 往纸质表新增数据
     * @param obj
     */
    public void insertPaperPosition(EmrPaperPositionInfo obj){
        SimpleDateFormat fmt = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        String time = fmt.format(new Date());
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String currentUserName = commomService.getCurrentUserName(request);
        try {
            long id = IDHelper.NewID();
            obj.setId(id);
        } catch (Exception e) {
            e.printStackTrace();
            ExceptionPrintUtil.printException(e);
        }
        obj.setUpdateUserName(currentUserName);
        obj.setUpdateTime(time);
        obj.setCreateUserName(currentUserName);
        obj.setCreateTime(time);
        paperPositionInfoMapper.insertSelective(obj);
    }

    public void updatePaperPostion(EmrPaperPositionInfo obj) {
        if(StringUtils.isNotBlank(obj.getPaperPosition()) || StringUtils.isNotBlank(obj.getPaperPosition1()) || StringUtils.isNotBlank(obj.getPaperPosition2()) || StringUtils.isNotBlank(obj.getPaperPosition3()) || StringUtils.isNotBlank(obj.getPaperPosition4())) {
            //处理纸质信息值带横杠的拆横杠，多个值的用横杠拼接
            handlePaperPostion(obj);
            //查询是否存在，存在则更新
            EmrPaperPositionInfo emrPaperPositionInfo = paperPositionInfoMapper.selectPaperPositionInfoByPatientId(obj.getPatientId());
            if (null != emrPaperPositionInfo) {
                emrPaperPositionInfo.setPaperPosition(obj.getPaperPosition());
                emrPaperPositionInfo.setPaperPosition1(obj.getPaperPosition1());
                emrPaperPositionInfo.setPaperPosition2(obj.getPaperPosition2());
                emrPaperPositionInfo.setPaperPosition3(obj.getPaperPosition3());
                emrPaperPositionInfo.setPaperPosition4(obj.getPaperPosition4());
                updateByPrimaryKeySelectiveUpdatePaperPosition(emrPaperPositionInfo);
            } else {
                insertPaperPosition(obj);
            }
        }
    }
}
