package com.emr.service.ipml;

import com.emr.dao.EmrHolidaySetMapper;
import com.emr.dao.Emr_DictionaryMapper;
import com.emr.dao.RecordMapper;
import com.emr.entity.*;
import com.emr.service.Archive_MasterService;
import com.emr.util.Msg;
import com.emr.vo.*;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.util.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.*;

@Service
public class RecordService {
    @Autowired
    private RecordMapper recordMapper;
    @Autowired
    private Emr_DictionaryMapper dictionaryMapper;
    @Autowired
    private CommomService commomService;
    @Autowired
    private Archive_MasterService archiveMasterService;
    @Autowired
    private EmrOvertimeSetService overtimeSetService;
    @Autowired
    private EmrHolidaySetMapper holidaySetMapper;
    @Autowired
    private ZdAssortService assortService;
    @Autowired
    private ArchiveOtherExtService otherExtService;
    /**
     * 病案提交统计
     * @return
     */
    public List<SubmitRecord> selectUnCountGroupDept(RecordSearch recordSearch, HttpServletRequest request) throws Exception{
        /*Integer infoId = recordSearch.getInfoId();
        if(null != infoId){
            //有按审核角色查询
            if(infoId < 5){
                //医生支线
                recordSearch.setOperRole(1);
            }else if(infoId == 5 || infoId == 6){
                //护士支线
                recordSearch.setOperRole(2);
            }else if(infoId == 8){
                recordSearch.setOperRole(4);
            }
        }*/
        //按科室查询总出院人数
        List<SubmitRecord> unSubmitRecords = recordMapper.selectUnCountGroupDept(recordSearch);
        if (!CollectionUtils.isEmpty(unSubmitRecords)) {
            if(null != recordSearch.getFlag() && recordSearch.getFlag() == 2){
                //过滤超期
                unSubmitRecords = selectOvertimeRecord(unSubmitRecords);
            }
            //按科室填充数据
            unSubmitRecords = selectUnSubmitCountGroupDept(unSubmitRecords);
            //填充各个角色数据、主要填充已提交的
            selectList(unSubmitRecords);
        }
        //request.getSession().setAttribute("unSubmitRecords",unSubmitRecords);
        return unSubmitRecords;
    }

    /**
     * 过滤未过期的
     * @param unSubmitRecords
     * @return
     */
    private List<SubmitRecord> selectOvertimeRecord(List<SubmitRecord> unSubmitRecords) throws Exception{
        //查询超期参数
        EmrOvertimeSet overtimeSet = overtimeSetService.selectByPrimaryKey(1);
        //获取未超期列表
        List<EmrHolidaySet> unHolidays = holidaySetMapper.selectNotHolidayList();
        Iterator<SubmitRecord> iterator = unSubmitRecords.iterator();
        //获取非死亡工作日
        String date = unHolidays.get(overtimeSet.getDays()).getDate();
        //获取非死亡工作日
        String deadDate = unHolidays.get(overtimeSet.getDeadDays()).getDate();
        //定义格式化日期
        SimpleDateFormat fmt = new SimpleDateFormat("yyyy-MM-dd");
        while (iterator.hasNext()) {
            SubmitRecord record = (SubmitRecord) iterator.next();
            //获取是否死亡
            String dischargeDisposition = record.getDischargeDisposition();
            String dischargeDateTime = record.getDischargeDateTime();
            long time = fmt.parse(dischargeDateTime).getTime();
            if(StringUtils.isNotBlank(dischargeDisposition) && "5".equals(dischargeDisposition)){
                //死亡
                if(time >= fmt.parse(deadDate).getTime()){
                    iterator.remove();
                }
            }else{
                //非死亡
                if(time >= fmt.parse(date).getTime()){
                    iterator.remove();
                }
            }
        }
        return unSubmitRecords;
    }

    /**
     * 按科室查询总出院人数
     * @param unSubmitRecords 出院列表
     * @return
     */
    private List<SubmitRecord> selectUnSubmitCountGroupDept(List<SubmitRecord> unSubmitRecords) {
        //查询科室集合
        List<Emr_Dictionary> depts = dictionaryMapper.selectAllByParentId("dept_code", "CODE");
        //医生未提交状态码集合
        String[] unDoctorSubmitArr = {"1","16","48","512","256","272","304","768"};
        List<String> unDoctorSubmitStr = Arrays.asList(unDoctorSubmitArr);
        //医生质控员未提交状态码集合
        String[] unDoctorCheckArr = {"2","18","50","514"};
        List<String> unDoctorCheckStr = Arrays.asList(unDoctorCheckArr);
        //科主任未提交状态码集合
        String[] unDirectorCheckArr = {"6","22","54","518"};
        List<String> unDirectorCheckStr = Arrays.asList(unDirectorCheckArr);
        //护士未提交状态码集合
        String[] unNurseSubmitArr = {"1","2","6","14","256","512","514","518","526","768"};
        List<String> unNurseSubmitStr = Arrays.asList(unNurseSubmitArr);
        //护士质控员未提交状态码集合
        String[] unNurseCheckArr = {"16","18","22","30","272"};
        List<String> unNurseCheckStr = Arrays.asList(unNurseCheckArr);
        //护士长未提交状态码集合
        String[] unHeadNurseArr = {"62"};
        List<String> unHeadNurseStr = Arrays.asList(unHeadNurseArr);
        //病案室未提交状态码集合
        String[] unRecordRoomArr = {"64","1024"};
        List<String> unRecordRoomStr = Arrays.asList(unRecordRoomArr);
        List<SubmitRecord> list = new ArrayList<>();
        for(Emr_Dictionary dept : depts) {
            int allCount = 0;
            //定义医生未提交数量
            int unDoctorSubmitCount = 0;
            int unDoctorCheckCount = 0;
            int unDirectorCheckCount = 0;
            int unNurseSubmitCount = 0;
            int unNurseCheckCount = 0;
            int unHeadNurseCount = 0;
            int unRecordRoomCount = 0;
            for (SubmitRecord obj : unSubmitRecords) {
                if(dept.getCode().equals(obj.getDeptName())) {
                    allCount++;
                    //判断属于哪个审核角色未提交状态
                    if (unDoctorSubmitStr.contains(obj.getArchivestate())) {
                        unDoctorSubmitCount++;
                    }
                    if (unDoctorCheckStr.contains(obj.getArchivestate())) {
                        unDoctorCheckCount++;
                    }
                    if (unDirectorCheckStr.contains(obj.getArchivestate())) {
                        unDirectorCheckCount++;
                    }
                    if (unNurseSubmitStr.contains(obj.getArchivestate())) {
                        unNurseSubmitCount++;
                    }
                    if (unNurseCheckStr.contains(obj.getArchivestate())) {
                        unNurseCheckCount++;
                    }
                    if (unHeadNurseStr.contains(obj.getArchivestate())) {
                        unHeadNurseCount++;
                    }
                    if (unRecordRoomStr.contains(obj.getArchivestate())) {
                        unRecordRoomCount++;
                    }
                }
            }
            if(allCount != 0){
                SubmitRecord record = new SubmitRecord();
                record.setCount(allCount);
                record.setUnDoctorSubmitCount(unDoctorSubmitCount);
                record.setUnDoctorCheckCount(unDoctorCheckCount);
                record.setUnDirectorCheckCount(unDirectorCheckCount);
                record.setUnNurseSubmitCount(unNurseSubmitCount);
                record.setUnNurseCheckCount(unNurseCheckCount);
                record.setUnHeadNurseCount(unHeadNurseCount);
                record.setUnRecordRoomCount(unRecordRoomCount);
                record.setDeptNameCn(dept.getName());
                record.setDeptName(dept.getCode());
                list.add(record);
            }
        }
        return list;
    }
    /*public List<SubmitRecord> selectUnCountGroupDept(RecordSearch recordSearch){
        Integer infoId = recordSearch.getInfoId();
        if(null != infoId){
            //有按审核角色查询
            if(infoId < 5){
                //医生支线
                recordSearch.setOperRole(1);
            }else if(infoId == 5 || infoId == 6){
                //护士支线
                recordSearch.setOperRole(2);
            }else if(infoId == 8){
                recordSearch.setOperRole(4);
            }
        }
        //按科室查询总出院人数
        List<SubmitRecord> unSubmitRecords = recordMapper.selectUnCountGroupDept(recordSearch);
        //填充各个角色数据
        selectList(unSubmitRecords);
        return unSubmitRecords;
    }*/

    /**
     * 填充集合数据
     * @param unSubmitRecords
     */
    private void selectList(List<SubmitRecord> unSubmitRecords) {
        if(!CollectionUtils.isEmpty(unSubmitRecords)) {
            for (SubmitRecord obj : unSubmitRecords) {
                int count = obj.getCount();
                if (count > 0) {
                    //医生提交== 总数 - 医生未提交
                    int doctorSubmitCount = count - obj.getUnDoctorSubmitCount();
                    obj.setDoctorSubmitCount(doctorSubmitCount);
                    obj.setDoctorSubmitPercent(Math.round(doctorSubmitCount * 100 / count) + "%");

                    //医生质控员提交= 医生提交 - 医生质控员未提交
                    int doctorCheckCount = doctorSubmitCount - obj.getUnDoctorCheckCount();
                    obj.setDoctorCheckCount(doctorCheckCount);
                    //医生质控员提交率
                    int doctorCheckPercent = 0;
                    if(doctorSubmitCount > 0){
                        doctorCheckPercent = Math.round(doctorCheckCount * 100 / doctorSubmitCount);
                    }
                    obj.setDoctorCheckPercent(doctorCheckPercent + "%");
                    //科主任提交= 医生质控员提交 - 主任未提交
                    int directorCheckCount = doctorCheckCount - obj.getUnDirectorCheckCount();
                    obj.setDirectorCheckCount(directorCheckCount);
                    //科主任提交率
                    int directorCheckPercent = 0;
                    if(doctorCheckCount > 0){
                        directorCheckPercent = Math.round(directorCheckCount * 100 / doctorCheckCount);
                    }
                    obj.setDirectorCheckPercent(directorCheckPercent + "%");
                    //护士提交= 总数 - 护士未提交
                    int nurseSubmitCount = count - obj.getUnNurseSubmitCount();
                    obj.setNurseSubmitCount(nurseSubmitCount);
                    obj.setNurseSubmitPercent(Math.round(nurseSubmitCount * 100 / count) + "%");
                    //护士质控员提交= 护士提交-护士质控员未提交
                    int nurseCheckCount = nurseSubmitCount - obj.getUnNurseCheckCount();
                    obj.setUnNurseCheckCount(nurseSubmitCount - nurseCheckCount);
                    obj.setNurseCheckCount(nurseCheckCount);
                    //护士质控员提交率
                    int nurseCheckPercent = 0;
                    if(nurseSubmitCount > 0){
                        nurseCheckPercent = Math.round(nurseCheckCount * 100 / nurseSubmitCount);
                    }
                    obj.setNurseCheckPercent(nurseCheckPercent + "%");

                    //护士长提交= 科主任或护士质控员提交的数 - 护士长未提交
                    //定义护士长前提交数量,医生和护士之前的最后一个节点都提交的数量。取科主任和护士质控员中小的值
                    int beforeHeadNurse = directorCheckCount < nurseCheckCount ? directorCheckCount : nurseCheckCount;
                    int headNurseCount = beforeHeadNurse - obj.getUnHeadNurseCount();
                    //护士长未提交等于提交上来的减去护士上长自己提交的
                    obj.setHeadNurseCount(headNurseCount);
                    //定义护士长提交率
                    int headNursePercent = 0;
                    if(beforeHeadNurse > 0){
                        headNursePercent = Math.round(headNurseCount * 100 / beforeHeadNurse);
                    }
                    obj.setHeadNursePercent(headNursePercent + "%");
                }
            }
        }
    }

    /**
     * 病案归档统计
     * @return
     */
    public SubmitRecordStatistics selectCount(RecordSearch recordSearch) {
        SubmitRecordStatistics record = recordMapper.selectCount(recordSearch);
        //计算百分比并赋值
        setStatisticsPercent(record);
        return record;
    }

    /**
     * 病案提交明细,按时间统计
     * @return
     */
    public List<SubmitRecordStatistics> getSignedRecordList(RecordSearch recordSearch){
        //按日期查询
        List<SubmitRecordStatistics> dateList = recordMapper.selectGroupDate(recordSearch);
        if(!CollectionUtils.isEmpty(dateList)){
            //查询属于需要校验完整的带分类的集合
            //设置查询的前后日期
            recordSearch.setEndDate(dateList.get(0).getDisDate());
            recordSearch.setStartDate(dateList.get(dateList.size()-1).getDisDate());
            List<Archive_Master_Vo> masterVoList = recordMapper.selectAllWithCompleteAssort(recordSearch);
            //组织masterId分组数据
            //List<Archive_Master_Vo> masterIdVoList = selectGroupDataByMasterId(masterVoList);
            //按日期分组和masterId分组组织已提交和已归档的数量
            selectGroupDateByMasterIdList(dateList,masterVoList);
            //转换百分比并有按科室查询时转换科室名称
            setDeptNameAndPercent(recordSearch, dateList);
        }
        return dateList;
    }

    /**
     * 按日期分组和masterId分组组织已提交和已归档的数量
     * @param dateSet 日期分组
     * @param masterIdVoList masterId分组数据
     * @return
     */
    private void selectGroupDateByMasterIdList(List<SubmitRecordStatistics> dateSet, List<Archive_Master_Vo> masterIdVoList) {
        for(SubmitRecordStatistics submitRecordStatistics : dateSet){
            //定义该日期总数量
            int allCount = 0;
            //定义已提交份数
            int count1 = 0;
            //定义已归档份数
            int count2 = 0;
            //定义完整份数
            int count3 = 0;
            for (Archive_Master_Vo obj : masterIdVoList){
                if(obj.getDischargeDateTime().equals(submitRecordStatistics.getDisDate())){
                    allCount++;
                    //获取状态码
                    String archivestate = obj.getArchivestate();
                    if("64".equals(archivestate) || "128".equals(archivestate) || "1024".equals(archivestate)){
                        count1++;
                    }
                    if("128".equals(archivestate)){
                        count2++;
                    }
                    if("完整".equals(obj.getLockinfo())){
                        count3++;
                    }
                }
            }
            submitRecordStatistics.setAllCount(allCount);
            submitRecordStatistics.setCount1(count1);
            submitRecordStatistics.setCount2(count2);
            submitRecordStatistics.setCount3(count3);
        }
    }

    /**
     * 组织masterId分组数据,带状态码，日期，是否完整
     * @param masterVoList
     * @return
     */
    private List<Archive_Master_Vo> selectGroupDataByMasterId(List<Archive_Master_Vo> masterVoList) {
        //查询需要校验的分类集合
        List<Zd_Assort> assortList = assortService.selectAllByIsCheck();
        //查询ext表的不完整的集合
        List<ArchiveOtherExt> archiveOtherExts = otherExtService.selectNotCompleteMidList();
        //组织masterId分组数据,带状态码，日期，是否完整
        for(Archive_Master_Vo obj : masterVoList) {
            //定义ext表中不存在
            boolean exist = false;
            if (!CollectionUtils.isEmpty(archiveOtherExts)) {
                for (ArchiveOtherExt archiveOtherExt : archiveOtherExts) {
                    if (obj.getId().equals(archiveOtherExt.getId())) {
                        exist = true;
                        break;
                    }
                }
            }
            //如果数量相等，且在ext表中不存在则isSuccess置为1
            if (obj.getCount() == assortList.size() && !exist) {
                obj.setIsSuccess("1");
            }
        }
        return masterVoList;
    }

    /**
     * 转换百分比并有按科室查询时转换科室名称
     * @param recordSearch
     * @param list
     */
    private void setDeptNameAndPercent(RecordSearch recordSearch, List<SubmitRecordStatistics> list) {
        String deptNameCn = "全部";
        String deptName = recordSearch.getDeptName();
        if(StringUtils.isNotBlank(deptName)){
            Emr_Dictionary dictionary = new Emr_Dictionary();
            dictionary.setCode(deptName);
            //转换科室
            List<Emr_Dictionary> dictionaries = dictionaryMapper.dicByClo(dictionary);
            if(!CollectionUtils.isEmpty(dictionaries)){
                deptNameCn = dictionaries.get(0).getName();
            }
        }
        if(!CollectionUtils.isEmpty(list)){
            for (SubmitRecordStatistics obj:list) {
                //计算百分比并赋值
                setStatisticsPercent(obj);
                obj.setDeptNameCn(deptNameCn);
            }
        }
    }

    /**
     * 设置提交报表百分比并赋值
     * @param record
     */
    private void setStatisticsPercent(SubmitRecordStatistics record) {
        Integer count = record.getAllCount();
        if (count != 0) {
            record.setCount1Percent(Math.round(record.getCount1() * 100 / count) + "%");
            record.setCount2Percent(Math.round(record.getCount2() * 100 / count) + "%");
            record.setCount3Percent(Math.round(record.getCount3() * 100 / count) + "%");
        } else {
            record.setCount1Percent("0%");
            record.setCount2Percent("0%");
            record.setCount3Percent("0%");
        }
    }

    /**
     * 设置超期列表提交率
     * @param recordSearch
     * @return
     */
    public SubmitRecordStatistics selectOvertimeCount(RecordSearch recordSearch,HttpServletRequest request) throws Exception{
        //查询超期记录数
        List<SubmitRecord> submitRecords = selectUnCountGroupDept(recordSearch,request);
        //定义接收返回对象
        SubmitRecordStatistics recordStatistics = new SubmitRecordStatistics();
        if(!CollectionUtils.isEmpty(submitRecords)){
            //计算
            //定义总超期数
            int allCount = 0;
            //定义医生总超期数
            int count1 = 0;
            int count2 = 0;
            int count3 = 0;
            int count4 = 0;
            int count5 = 0;
            int count6 = 0;
            for(SubmitRecord record : submitRecords){
                allCount += record.getCount();
                count1 += record.getDoctorSubmitCount();
                count2 += record.getDoctorCheckCount();
                count3 += record.getHeadNurseCount();
                count4 += record.getNurseSubmitCount();
                count5 += record.getNurseCheckCount();
                count6 += record.getHeadNurseCount();
            }
            recordStatistics.setAllCount(allCount);
            recordStatistics.setCount1Percent(Math.round(count1 * 100 / allCount) + "%");
            recordStatistics.setCount2Percent(Math.round(count2 * 100 / allCount) + "%");
            recordStatistics.setCount3Percent(Math.round(count3 * 100 / allCount) + "%");
            recordStatistics.setCount4Percent(Math.round(count4 * 100 / allCount) + "%");
            recordStatistics.setCount5Percent(Math.round(count5 * 100 / allCount) + "%");
            recordStatistics.setCount6Percent(Math.round(count6 * 100 / allCount) + "%");
            int count7 = allCount - count6;
            recordStatistics.setCount7Percent(Math.round(count7 * 100 / allCount) + "%");
        }
        return recordStatistics;
    }

    /**
     * 科室名称编码转名称
     * @param dictionaries 科室字典列表
     * @param vo 患者记录对象
     */
    private void deptNameCode2Name(List<Emr_Dictionary> dictionaries, Archive_Master_Vo vo) {
        if(!CollectionUtils.isEmpty(dictionaries)) {
            //在院科室
            String deptAdmissionTo = vo.getDeptAdmissionTo();
            //出院科室
            String deptName = vo.getDeptName();
            //住院科室
            for (Emr_Dictionary dictionaryTemp : dictionaries) {
                String name = dictionaryTemp.getName();
                if (StringUtils.isNotBlank(deptAdmissionTo) && deptAdmissionTo.equals(dictionaryTemp.getCode())) {
                    vo.setDeptAdmissionTo(name);
                }
                if (StringUtils.isNotBlank(deptName) && deptName.equals(dictionaryTemp.getCode())) {
                    vo.setDeptName(name);
                }
            }
        }
    }

    /**
     * 按科室待审核角色病案超时分页查询
     * @param recordSearch
     * @return
     */
    public List<Archive_Master_Vo> selectRecordGroupDeptAndRole(RecordSearch recordSearch) throws Exception{
        List<Archive_Master_Vo> list = recordMapper.selectRecordGroupDeptAndRole(recordSearch);
        if(!CollectionUtils.isEmpty(list)){
            List<User> userList = commomService.getUserList();
            //查询科室集合
            Emr_Dictionary dictionary = new Emr_Dictionary();
            dictionary.setTypecode("dept_code");
            List<Emr_Dictionary> dictionaries = dictionaryMapper.dicByTypeCode(dictionary);
            for (Archive_Master_Vo obj : list) {
                //转换科室
                deptNameCode2Name(dictionaries, obj);
                //转换工号
                if(!CollectionUtils.isEmpty(userList)) {
                    for (User user : userList) {
                        if (StringUtils.isNotBlank(obj.getDoctorInCharge()) && obj.getDoctorInCharge().equals(user.getName())) {
                            obj.setDoctorInChargeCode(user.getUserName());
                            break;
                        }
                    }
                }
                //转换状态
                String archivestate = obj.getArchivestate();
                if (StringUtils.isNotBlank(archivestate)) {
                    Msg role = archiveMasterService.getRole();
                    //转换中文状态
                    boolean numeric = archiveMasterService.isNumeric(archivestate);
                    if (numeric) {
                        Integer status = Integer.valueOf(archivestate);
                        archivestate = EnumVerify.DocState.GetStepName(status, role.getCode());
                        obj.setStatus(archivestate);
                    }
                }
            }
        }
        return list;
    }
}
