package com.emr.service.ipml;

import com.emr.dao.RecordMapper;
import com.emr.entity.EmrOvertimeSet;
import com.emr.entity.Power_User;
import com.emr.util.HttpClientUtils;
import com.emr.vo.RecordSearch;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.*;

/**
 * @ProjectName:
 * @Description:
 * @Param 传输参数
 * @Return
 * @Author: 曾文和
 * @CreateDate: 2020/11/10 14:20
 * @UpdateUser: 曾文和
 * @UpdateDate: 2020/11/10 14:20
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
@Service
public class SendMessageService {
    @Value("${overDateSet}")
    private String overDateSet;
    @Value("${POWER_URLHEAD}")
    private String POWER_URLHEAD;
    @Autowired
    private EmrOvertimeSetService overtimeSetService;
    @Autowired
    private RecordMapper recordMapper;
    @Autowired
    private CommomService commomService;
    /**
     * 根据角色Id获取角色名称和角色称呼
     * @param roleId
     * @return list list.get(0) = 角色名称 list.get(1) = 角色称呼 list.get(2) = infoId
     */
    public List<String> selectInfoIdAndRoleName(int roleId) {
        //roleId:1:医生,2:护士，3:病案室人员,5:医生质控员，6:科主任，7:护士长,8:护士质控员
        //infoId:2：医生；3：医生质控员；4：科主任；5：护士；6：护士质控员;7：护士长；8：病案室人员；
        List<String> list = new LinkedList<>();
        switch (roleId){
            case 1:
                list.add("医生");
                list.add("医生待审核");
                list.add("2");
                break;
            case 5:
                list.add("医生");
                list.add("质控员待审核");
                list.add("3");
                break;
            case 6:
                list.add("主任");
                list.add("科主任待审核");
                list.add("4");
                break;
            case 2:
                list.add("护士");
                list.add("护士待审核");
                list.add("5");
                break;
            case 8:
                list.add("护士");
                list.add("质控员待审核");
                list.add("6");
                break;
            case 7:
                list.add("护士长");
                list.add("护士长待审核");
                list.add("7");
                break;
            case 3:
                list.add("病案室人员");
                list.add("病案室人员待审核");
                list.add("8");
                break;
            default:
                list.add("无效角色");
                list.add("无效角色待审核");
                list.add("1");
        }
        return list;
    }

    /**
     * 组织发送消息
     * @param name 姓名
     * @param count 待审核份数
     * @param roleList 根据角色Id获取角色名称和角色称呼 list list.get(0) = 角色名称 list.get(1) = 角色称呼
     * @return
     */
    public String selectContentByRole(String name,Integer count,List<String> roleList){
        return name + roleList.get(0) + ",您有"+count+"份日期已超过预超期日期是"+roleList.get(1)+"状态的病历，请及时查阅";
    }

    /**
     * 调用权限系统发送消息接口
     * @param userId 用户Id
     * @param content 发送内容
     */
    public void sendMessage(Integer userId, String content,String typeFlag) {
        Map<String, String> map = new HashMap<>();
        map.put("title", "待审核病历即将超期通知");
        map.put("content", content);
        map.put("sysFlag", "emr_medical_record");
        map.put("userId", userId + "");
        map.put("typeFlag", typeFlag);
        //请求power系统发送信息
        HttpClientUtils.doGet(POWER_URLHEAD + "/font/sendMessage", map);
    }

    /**
     * 根据登录用户发送即将超期待审核提醒
     * @param request
     */
    public void sendMessageForSubmit(HttpServletRequest request) throws Exception {
        //获取登录用户
        Power_User user = (Power_User)request.getSession().getAttribute("CURRENT_USER");
        //定义需要发送通知的角色id集合
        Integer[] roleIdArr = {1,2,3,5,6,7,8};
        int roleId = user.getRoleId();
        //判断是否需要发送通知
        int flag = Arrays.binarySearch(roleIdArr,roleId);
        if(flag >= 0) {
            //获取deptCode
            String deptCode = commomService.selectDeptCodeByUser(user.getUserName());
            if(StringUtils.isNoneBlank(deptCode) && !"1".equals(deptCode)) {
                //组织即将到期的待审核病历数的查询条件
                RecordSearch recordSearch = selectRecordSearch();
                //非病案室人员根据科室查
                if(user.getRoleId() != 3) {
                    recordSearch.setDeptName(deptCode);
                }
                //是医生的话明细到主管医生份数
                if(user.getRoleId().equals(1)){
                    recordSearch.setDoctorInCharge(user.getUserPosition());
                }
                //获取角色名称与角色称呼、infoId
                List<String> roleList = selectInfoIdAndRoleName(user.getRoleId());
                int infoId = Integer.valueOf(roleList.get(2));
                recordSearch.setInfoId(infoId);
                //查询该用户即将到期的待审核病历数
                int count = recordMapper.selectCountOverTimeByRoleId(recordSearch);
                if(count > 0) {
                    //组织通知内容 XXX医生或护士，您有X份病历是XX（角色）待审核状态，请及时查阅
                    String content = selectContentByRole(user.getUserPosition(), count, roleList);
                    //调用权限系统发送消息接口
                    sendMessage(user.getUserId(), content,"sendOverTimeMessage_"+infoId);
                }
            }
        }
    }

    /**
     * 组织即将过期的待审核份数的查询条件
     * @return
     */
    public RecordSearch selectRecordSearch(){
        RecordSearch recordSearch = new RecordSearch();
        //设置查询超期标识
        recordSearch.setFlag(2);
        //设置起始时间
        recordSearch.setStartDate(overDateSet);
        //设置提前提醒天数
        EmrOvertimeSet emrOvertimeSet = overtimeSetService.selectByPrimaryKey(1);
        recordSearch.setSurplus(emrOvertimeSet.getSurplusDays());
        return recordSearch;
    }
}
