package com.emr.service.ipml;

import com.emr.dao.*;
import com.emr.entity.Archive_Master_Vo;
import com.emr.entity.CollectorStatus;
import com.emr.entity.OffsetLimitPage;
import com.emr.vo.*;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.util.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * @ProjectName:
 * @Description:
 * @Param 传输参数
 * @Return
 * @Author: 曾文和
 * @CreateDate: 2020/1/8 15:54
 * @UpdateUser: 曾文和
 * @UpdateDate: 2020/1/8 15:54
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
@Service
@Transactional
public class StatisticsService {
    @Autowired
    private StatisticsMapper statisticsMapper;
    @Autowired
    private TUuPrintMapper uuPrintMapper;
    @Autowired
    private TUuInfoMapper uuInfoMapper;
    @Autowired
    private RecordStatisticsMapper recordStatisticsMapper;
    @Autowired
    private CommomService commomService;
    @Autowired
    private CollectorStatusMapper collectorStatusMapper;
    //终审按天统计
    public List<FinalAndFirstStatistics> getFinalStatistics(Integer offset, Integer limit, String startDate, String endDate,String sql) throws Exception{
        if(null != offset && null != limit){
            PageHelper.offsetPage(offset, limit);
        }
        List<FinalAndFirstStatistics> list = statisticsMapper.finalStatistics(startDate, endDate,sql);
        //转换姓名
        List<User> userList = commomService.getUserList();
        if(!CollectionUtils.isEmpty(list) && !CollectionUtils.isEmpty(userList)) {
            for (FinalAndFirstStatistics statistics : list) {
                for (User user : userList) {
                    if (StringUtils.isNotBlank(statistics.getCheckCode()) && statistics.getCheckCode().equals(user.getUserName())) {
                        statistics.setCheckName(user.getName());
                        break;
                    }
                }
            }
        }
        return list;
    }


    //初审按天统计
    public List<FinalAndFirstStatistics> getFirstStatistics(Integer offset, Integer limit, String startDate, String endDate,String sql) throws Exception{
        if(null != offset && null != limit){
            PageHelper.offsetPage(offset, limit);
        }
        List<FinalAndFirstStatistics> list = statisticsMapper.firstStatistics(startDate, endDate,sql);
        //转换姓名
        List<User> userList = commomService.getUserList();
        if(!CollectionUtils.isEmpty(list) && !CollectionUtils.isEmpty(userList)) {
            for (FinalAndFirstStatistics statistics : list) {
                for (User user : userList) {
                    if (statistics.getCheckCode().equals(user.getUserName())) {
                        statistics.setCheckName(user.getName());
                        break;
                    }
                }
            }
        }
        return list;
    }

    /**
     * 终审审核明细
     * @param offset
     * @param limit
     * @param disStartDate
     * @param disEndDate
     * @param archiveMasterVo
     * @return
     */
    public OffsetLimitPage getStatisticsDetail(Integer offset, Integer limit,String disStartDate, String disEndDate, Archive_Master_Vo archiveMasterVo,Integer flag) throws Exception{
        if(null != offset && null != limit){
            PageHelper.offsetPage(offset, limit);
        }
        List<Archive_Master_Vo> list = getDetailList(disStartDate, disEndDate, archiveMasterVo,flag,null);
        return new OffsetLimitPage((Page) list);
    }

    public List<Archive_Master_Vo> getDetailList(String disStartDate, String disEndDate, Archive_Master_Vo archiveMasterVo,Integer flag,String sql) throws Exception{
        //转换姓名
        List<User> userList = commomService.getUserList();
        String checkDoctor = archiveMasterVo.getCheckDoctor();
        //模糊搜索审核姓名转换工号
        if(StringUtils.isNoneBlank(checkDoctor)){
            StringBuilder checkNames = new StringBuilder();
            for(User user:userList){
                if(StringUtils.isNoneBlank(user.getName()) && user.getName().contains(checkDoctor)){
                    checkNames.append("'").append(user.getUserName()).append("',");
                }
            }
            if(StringUtils.isNotBlank(checkNames.toString())){
                checkNames = new StringBuilder(checkNames.substring(0, checkNames.length() - 1));
                archiveMasterVo.setCheckDoctor(checkNames.toString());
            }else{
                archiveMasterVo.setCheckDoctor("'" + checkDoctor + "'");
            }
        }
        List<Archive_Master_Vo> list = statisticsMapper.statistics(disStartDate,disEndDate,archiveMasterVo,flag,sql);
        if(!CollectionUtils.isEmpty(list) && !CollectionUtils.isEmpty(userList)) {
            for (Archive_Master_Vo statistics : list) {
                for (User user : userList) {
                    if (StringUtils.isNotBlank(statistics.getCheckDoctor()) && statistics.getCheckDoctor().equals(user.getUserName())) {
                        statistics.setCheckName(user.getName());
                    }
                    if (StringUtils.isNotBlank(statistics.getCheckedDoctor()) && statistics.getCheckedDoctor().equals(user.getUserName())) {
                        statistics.setCheckedName(user.getName());
                    }
                }
            }
        }
        return list;
    }

    //复印记录报表
    public List<TUuPrintVo> getPrintCount(TUuPrintSearch search){
        return uuPrintMapper.getPrintCount(search);
    }

    //复印记录明细
    public List<TUuPrintVo> getPrintInfo(TUuPrintSearch search){
        List<TUuPrintVo> list = uuPrintMapper.getPrintInfo(search);
        if(null != list && !list.isEmpty()){
            SimpleDateFormat fmt = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            for(TUuPrintVo printVo : list){
                Date cpytime = printVo.getCpytime();
                printVo.setPrintTime(fmt.format(cpytime));
            }
        }
        return list;
    }

    //扫描记录报表
    public List<TUuInfoVo> getScanCount(TUuPrintSearch search) throws Exception{
        List<TUuInfoVo> countlist = uuInfoMapper.getScanCount(search);
        List<TUuInfoVo> recordCountlist = uuInfoMapper.selectScanRecordCountGySysAndDate(search);
        if(!CollectionUtils.isEmpty(countlist)){
            for(TUuInfoVo obj : countlist){
                for(TUuInfoVo obj1 : recordCountlist){
                    if(StringUtils.isNotBlank(obj.getUuname())) {
                        if (obj.getUuname().equals(obj1.getUuname()) && obj.getUploaddatetime().equals(obj1.getUploaddatetime())) {
                            obj.setScanNums(obj1.getScanNums());
                            break;
                        }
                    }
                }
            }
        }
        return countlist;
    }
    //采集器状态
    public List<CollectorStatusVo> getCollectorStatus(CollectorStatus collectorStatus){
        List<CollectorStatusVo> list = collectorStatusMapper.getCollectorStatus(collectorStatus);
       //获取系统当前时间
        Long time = System.currentTimeMillis();
        //时间格式转换
        SimpleDateFormat format=new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        time -=20*1000*60;//在当前系统时间的基础上减去20分钟
        String currentTime = format.format(time);
        if (list!=null){
            for (CollectorStatusVo StatusList:list){
                String updateTime = StatusList.getUpdateTime();
                if(updateTime.compareTo(currentTime)>0){
                    StatusList.setCollectorStatus("正在运行");
                }else {
                    StatusList.setCollectorStatus("停止运行");
                }
                String c1 = StatusList.getC1();
                if(StringUtils.isNotBlank(c1)){
                    StatusList.setC1("采集任务已完成");
                }else {
                    StatusList.setC1("采集任务未完成");
                }
            }
        }
        return list;
    }

    private String setSearchName(TUuPrintSearch search, List<User> userList) {
        String name = "";
        //如果带姓名模糊查询，转换成工号
        if(StringUtils.isNotBlank(search.getName())){
            StringBuilder nameStr = new StringBuilder();
            for(User user : userList){
                if(user.getName().contains(search.getName())){
                    if(StringUtils.isNotBlank(nameStr)){
                        nameStr.append(",");
                    }
                    nameStr.append("'").append(user.getUserName()).append("'");
                }
            }
            if(StringUtils.isNotBlank(nameStr)){
                //如果不为空，带工号集合查询
                name = nameStr.toString();
            }else{
                //如果为空，查询无结果，name值带&查询
                name = "&";
            }
        }
        return name;
    }

    //扫描记录明细
    public List<TUuInfoVo> getScanInfo(TUuPrintSearch search){
        return uuInfoMapper.getScanInfo(search);
    }

    /**
     * 统计病案列表
     * @param recordStatistics
     * @return
     */
    public List<RecordStatistics> getRecordStatistics(RecordStatistics recordStatistics) {
        List<RecordStatistics> list = new Page<>();
        RecordStatistics record = new RecordStatistics();
        //设置总病历数
        int recordCount = recordStatisticsMapper.getRecordCount(recordStatistics);
        record.setAllRecordCount(recordCount);
        List<RecordStatistics> fileCountList = null;
        if(recordCount != 0) {
            //查询总文件数
            fileCountList = recordStatisticsMapper.getFileCount(recordStatistics);
        }
        //计算总文件数,总自动采集文件数，扫描上传文件数,计算页数
        setFileCounts(record, fileCountList,null,null);
        list.add(record);
        return list;
    }

    //计算总文件数,总自动采集文件数，扫描上传文件数
    private void setFileCounts(RecordStatistics record, List<RecordStatistics> fileCountList,String param,String paramFlag) {
        int allFileCount = 0;
        int autoFileCount = 0;
        int scanFileCount = 0;
        //页数计算
        int allPageCount = 0;
        int autoCollectionPageCount = 0;
        int manualScanPageCount = 0;
        if(!CollectionUtils.isEmpty(fileCountList) && (record.getAllRecordCount() == null || record.getAllRecordCount() != 0)){
            for(RecordStatistics recordTemp : fileCountList){
                //定义是否匹配标识
                boolean flag = false;
                if("deptName".equals(paramFlag)){
                    flag = recordTemp.getDeptName().equals(param);
                }else if("masterId".equals(paramFlag)){
                    flag = recordTemp.getMasterId().equals(param);
                }
                //匹配上或不需匹配时才参与计算
                if(flag || StringUtils.isBlank(paramFlag)) {
                    Integer fileCount = recordTemp.getAllFileCount();
                    //计算总文件数
                    allFileCount += fileCount;
                    String source = recordTemp.getSource();
                    //获取分类页数
                    Integer allPageCountTemp = recordTemp.getAllPageCount();
                    //总页数
                    allPageCount += allPageCountTemp;
                    if (StringUtils.isNotBlank(source) && "扫描上传".equals(source)) {
                        //计算扫描上传的文件数
                        scanFileCount += fileCount;
                        //计算扫描上传的页数
                        manualScanPageCount += allPageCountTemp;
                    } else {
                        //计算自动采集的文件数
                        autoFileCount += fileCount;
                        //计算自动采集的页数
                        autoCollectionPageCount += allPageCountTemp;
                    }
                }
            }
        }
        //设置总文件数
        record.setAllFileCount(allFileCount);
        //设置自动采集总文件数
        record.setAutoCollectionFileCount(autoFileCount);
        //设置扫描上传总文件数
        record.setManualScanFileCount(scanFileCount);
        //设置总页数
        record.setAllPageCount(allPageCount);
        //设置自动采集总页数
        record.setAutoCollectionPageCount(autoCollectionPageCount);
        //设置扫描上传总页数
        record.setManualScanPageCount(manualScanPageCount);
    }

    /**
     * 按科室分组统计病案
     * @param offset
     * @param limit
     * @param recordStatistics
     * @return
     */
    public List<RecordStatistics> getRecordStatisticsGroupDept(Integer offset,Integer limit,RecordStatistics recordStatistics) {
        if(null != offset && null != limit){
            PageHelper.offsetPage(offset, limit);
        }
        //按科室分组统计病案数
        List<RecordStatistics> list = recordStatisticsMapper.getRecordStatisticsGroupDept(recordStatistics);
        //按科室code匹配计算总文件数、自动采集文件数和扫描上传文件数
        if(!CollectionUtils.isEmpty(list)){
            //按科室和文件来源统计文件数
            List<RecordStatistics> fileList = recordStatisticsMapper.getFileStatisticsGroupDept(recordStatistics);
            for(RecordStatistics record : list){
                //获取科室code
                String deptName = record.getDeptName();
                //计算总文件数,总自动采集文件数，扫描上传文件数
                setFileCounts(record, fileList,deptName,"deptName");
            }
        }
        return list;
    }

    /**
     * 根据科室查询患者病案统计
     * @param offset
     * @param limit
     * @param recordStatistics
     * @return
     */
    public List<RecordStatistics> getRecordStatisticsByDeptName(Integer offset, Integer limit, RecordStatistics recordStatistics) {
        if(null != offset && null != limit){
            PageHelper.offsetPage(offset, limit);
        }
        //根据科室查询基本信息
        List<RecordStatistics> list = recordStatisticsMapper.getRecordStatisticsByDeptName(recordStatistics);
        //获取分页查询后的masterId集合
        if(!CollectionUtils.isEmpty(list)){
            //定义masterId集合
            StringBuilder masterIds = new StringBuilder();
            for (int i = 0; i < list.size(); i++) {
                String masterId = list.get(i).getMasterId();
                if(i == 0){
                    masterIds.append("'").append(masterId).append("'");
                }else{
                    masterIds.append(",'").append(masterId).append("'");
                }
            }
            //根据masterIds集合查询文件数集合
            List<RecordStatistics> fileCountList = recordStatisticsMapper.getFileCountByMasterIds(masterIds.toString());
            for(RecordStatistics record : list){
                String masterId = record.getMasterId();
                //计算总文件数,总自动采集文件数，扫描上传文件数
                setFileCounts(record, fileCountList,masterId,"masterId");
            }
        }
        return list;
    }
}

