package com.emr.service.ipml;

import com.emr.dao.*;
import com.emr.entity.*;
import com.emr.vo.*;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import org.apache.commons.lang3.StringUtils;
import org.apache.ibatis.annotations.Param;
import org.apache.shiro.util.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @ProjectName:
 * @Description:
 * @Param 传输参数
 * @Return
 * @Author: 曾文和
 * @CreateDate: 2021/7/28 14:22
 * @UpdateUser: 曾文和
 * @UpdateDate: 2021/7/28 14:22
 * @UpdateRemark: 更新说明
 * @Version: 1.0
 */
@Service
public class WorkHomeService {
    @Autowired
    private Archive_MasterMapper archiveMasterMapper;
    @Autowired
    private Emr_DictionaryMapper dictionaryMapper;
    @Autowired
    private RecordMapper recordMapper;
    @Autowired
    private EmrOvertimeSetService overtimeSetService;
    @Autowired
    private EmrHolidaySetMapper holidaySetMapper;
    @Autowired
    private Archive_DetailMapper archiveDetailMapper;
    @Autowired
    private ArchiveOtherExtMapper archiveOtherExtMapper;
    /**
     * 病案数据第一层表格
     * @param startDateTo
     * @param endDateTo
     * @return
     */
    public OffsetLimitPage recordDateOneTable(String startDateTo, String endDateTo) {
        //查询全部出院日期和状态
        List<Archive_Master> list= archiveMasterMapper.recordDateOneTable(startDateTo,endDateTo,null);
        //定义返回集合
        List<RecordStatisticsHome> returnList = new Page<>();
        //定义接收对象
        RecordStatisticsHome obj = new RecordStatisticsHome();
        obj.setName("全院");
        //定义出院人数
        int disCount = 0;
        //定义入院人数
        int admissCount = 0;
        //定义出科数量
        int sumbitCount = 0;
        //定义归档数量
        int recordCount = 0;
        if(!CollectionUtils.isEmpty(list)){
            for(Archive_Master archiveMaster : list){
                //判断出院、入院
                String disDate = archiveMaster.getDischargeDateTime();
                if(StringUtils.isNotBlank(disDate) && !"1801".equals(disDate.substring(0,4))){
                    disCount++;
                }else{
                    admissCount++;
                }
                //判断出科
                String archivestate = archiveMaster.getArchivestate();
                if(StringUtils.isNotBlank(archivestate)) {
                    String t1 = archiveMaster.getT1();
                    if ("64".equals(archivestate) || "128".equals(archivestate)) {
                        sumbitCount++;
                    }
                    //判断归档 待终审且已签收+已归档
                    if ("128".equals(archivestate) || ("64".equals(archivestate) && StringUtils.isNotBlank(t1) && !"1801-02-03".equals(t1.substring(0,10)))) {
                        recordCount++;
                    }
                }
            }
        }
        obj.setName("全院");
        obj.setDisCount(disCount);
        obj.setAdmissCount(admissCount);
        obj.setSumbitCount(sumbitCount);
        obj.setRecordCount(recordCount);
        if(disCount != 0) {
            obj.setRecordRate((recordCount * 100 / disCount)  + "%");
        }
        returnList.add(obj);
        return new OffsetLimitPage((Page) returnList);
    }

    /**
     * 病案数据第二层表格
     * @param offset
     * @param limit
     * @param startDateTo
     * @param endDateTo
     * @return
     */
    public OffsetLimitPage recordDataTwoTable(Integer offset, Integer limit, String startDateTo, String endDateTo) {
        PageHelper.offsetPage(offset, limit);
        //查询科室归档数量排名
        List<RecordStatisticsHome> returnList = archiveMasterMapper.recordDataTwoTable(startDateTo,endDateTo);
        if(!CollectionUtils.isEmpty(returnList)){
            //定义科室编号集合
            StringBuilder deptCodes = new StringBuilder();
            //查询科室集合
            Emr_Dictionary dictionary = new Emr_Dictionary();
            dictionary.setTypecode("dept_code");
            List<Emr_Dictionary> dictionaries = dictionaryMapper.dicByTypeCode(dictionary);
            for(RecordStatisticsHome obj : returnList){
                if(StringUtils.isNotBlank(deptCodes)){
                    deptCodes.append(",");
                }
                //出院科室
                String deptName = obj.getDeptName();
                deptCodes.append(deptName);
            }
            //查询全部出院日期和状态
            List<Archive_Master> list = archiveMasterMapper.recordDateOneTable(startDateTo,endDateTo,deptCodes.toString());
            for(RecordStatisticsHome vo : returnList){
                //出院科室
                String deptName = vo.getDeptName();
                //定义出院人数
                int disCount = 0;
                //定义入院人数
                int admissCount = 0;
                //定义归档数量
                int recordCount = 0;
                //定义出科数量
                int sumbitCount = 0;
                if(!CollectionUtils.isEmpty(list)){
                    for(Archive_Master archiveMaster : list){
                        if(StringUtils.isNotBlank(archiveMaster.getDeptName()) && archiveMaster.getDeptName().equals(deptName)){
                            //判断出院、入院
                            String disDate = archiveMaster.getDischargeDateTime();
                            if(StringUtils.isNotBlank(disDate) && !"1801".equals(disDate.substring(0,4))){
                                disCount++;
                            }else{
                                admissCount++;
                            }
                            String archivestate = archiveMaster.getArchivestate();
                            String t1 = archiveMaster.getT1();
                            //判断归档 待终审且已签收+已终审
                            if ("128".equals(archivestate) || ("64".equals(archivestate) && StringUtils.isNotBlank(t1) && !"1801-02-03".equals(t1.substring(0,10)))) {
                                recordCount++;
                            }
                            //判断出科
                            if ("128".equals(archivestate) || "64".equals(archivestate)) {
                                sumbitCount++;
                            }
                        }
                    }
                }
                vo.setDisCount(disCount);
                vo.setRecordCount(recordCount);
                vo.setAdmissCount(admissCount);
                vo.setSumbitCount(sumbitCount);
                //转换科室
                for (Emr_Dictionary dictionaryTemp : dictionaries) {
                    String name = dictionaryTemp.getName();
                    if (StringUtils.isNotBlank(deptName) && deptName.equals(dictionaryTemp.getCode())) {
                        vo.setName(name);
                    }
                }
            }
        }
        return new OffsetLimitPage((Page) returnList);
    }

    /**
     * 病案数据第三层表格
     * @param recordSearch
     * @return
     */
    public List<NotSubmitRecordVo> recordDataThreeTable(RecordSearch recordSearch) {
        List<SubmitRecord> list = recordMapper.selectUnCountGroupDeptForRemind(recordSearch);
        List<NotSubmitRecordVo> returnList = new ArrayList<>();
        if(!CollectionUtils.isEmpty(list)){
            SubmitRecord obj = list.get(0);
            //获取科室编号
            String deptName = obj.getDeptName();
            //获取科室名称
            String deptNameCn = obj.getDeptNameCn();
            int count2 = obj.getUnDoctorSubmitCount();
            if(count2 > 0){
                NotSubmitRecordVo vo = new NotSubmitRecordVo(deptName,deptNameCn,2,"医生未提交",count2);
                returnList.add(vo);
            }
            int count3 = obj.getUnDoctorCheckCount();
            if(count3 > 0){
                NotSubmitRecordVo vo = new NotSubmitRecordVo(deptName,deptNameCn,3,"医生质控员未提交",count3);
                returnList.add(vo);
            }
            int count4 = obj.getUnDirectorCheckCount();
            if(count4 > 0){
                NotSubmitRecordVo vo = new NotSubmitRecordVo(deptName,deptNameCn,4,"科主任未提交",count4);
                returnList.add(vo);
            }
            int count5 = obj.getUnNurseSubmitCount();
            if(count5 > 0){
                NotSubmitRecordVo vo = new NotSubmitRecordVo(deptName,deptNameCn,5,"护士未提交",count5);
                returnList.add(vo);
            }
            int count6 = obj.getUnNurseCheckCount();
            if(count6 > 0){
                NotSubmitRecordVo vo = new NotSubmitRecordVo(deptName,deptNameCn,6,"护士质控员未提交",count6);
                returnList.add(vo);
            }
            int count7 = obj.getUnHeadNurseCount();
            if(count7 > 0){
                NotSubmitRecordVo vo = new NotSubmitRecordVo(deptName,deptNameCn,7,"护士长未提交",count7);
                returnList.add(vo);
            }
            int count8 = obj.getUnRecordRoomCount();
            if(count8 > 0){
                NotSubmitRecordVo vo = new NotSubmitRecordVo(deptName,deptNameCn,8,"病案室未提交",count8);
                returnList.add(vo);
            }
        }
        return returnList;
    }

    /**
     * 病案室管理数据监控
     * @param startDateTo
     * @param endDateTo
     */
    public Map<String,String> medicalRoomManageData(String startDateTo, String endDateTo) throws Exception{
        //查询总数据 1.出院人数 2.待终审数量 3.已终审数量 4.已签收数量 5.已扫描数量 6.待终审已签收
        List<Integer> list = archiveMasterMapper.selectStatisticsCount(startDateTo, endDateTo);
        //组织返回数据
        //定义出院人数
        int disCount = 0;
        //定义待终审数量
        int headNurseCount = 0;
        //定义归档数量 已终审+待终审已签收
        int recordCount = 0;
        //定义签收数量
        int signCount = 0;
        //已扫描数量
        int isScanCount = 0;
        //超期病历
        int overTimeCount = 0;
        //预超期病历
        int reOverTimeCount = 0;
        //归档率 归档数量/出院人数
        int recordRate = 0;
        //签收率 已签收/总出院人数
        int signRate = 0;
        //扫描率 已扫描/已签收
        int scanRate = 0;
        Map<String, String> map = new HashMap<>();
        if (!CollectionUtils.isEmpty(list)) {
            disCount = list.get(0);
            headNurseCount = list.get(1);
            recordCount = list.get(2) + list.get(5);
            signCount = list.get(3);
            isScanCount = list.get(4);
            //查询超期参数
            EmrOvertimeSet overtimeSet = overtimeSetService.selectByPrimaryKey(1);
            //获取未超期列表
            List<EmrHolidaySet> unHolidays = holidaySetMapper.selectNotHolidayList();
            //获取非死亡工作日
            String date = unHolidays.get(overtimeSet.getDays()).getDate();
            //获取死亡工作日
            String deadDate = unHolidays.get(overtimeSet.getDeadDays()).getDate();
            //获取预超期非死亡工作日
            String reDate = unHolidays.get(overtimeSet.getDays() - overtimeSet.getSurplusDays()).getDate();
            //获取预超期死亡工作日
            String reDeadDate = unHolidays.get(overtimeSet.getDeadDays() - overtimeSet.getSurplusDays()).getDate();
            //查询超期与预超期数量集合 1.超期数量 2.预超期数量
            List<Integer> overTimeAndReOverTimeCountList = archiveMasterMapper.overTimeAndReOverTimeCount(date,deadDate,reDate,reDeadDate,startDateTo, endDateTo);
            overTimeCount = getOverTimeCount(startDateTo,endDateTo);
            reOverTimeCount = overTimeAndReOverTimeCountList.get(1);
        }
        //归档率 归档数量/出院人数
        if(disCount != 0){
            recordRate = recordCount * 100 / disCount;
        }
        map.put("recordRate",recordRate  + "%");
        //归档数量
        map.put("recordCount", recordCount + "");
        //待终审数量
        map.put("headNurseCount", headNurseCount + "");
        //出科数量 待终审数量+终审数量
        map.put("submitCount", (list.get(2) + headNurseCount) + "");
        //签收数量
        map.put("signCount", signCount + "");
        //待签收数量 总出院人数-已签收数量
        map.put("notSignCount", (disCount - signCount) + "");
        //签收率 已签收/总出院人数
        if(disCount != 0){
            signRate = signCount * 100 / disCount;
        }
        map.put("signRate",signRate + "%");
        //超期数量
        map.put("overTimeCount", overTimeCount + "");
        //预超期数量
        map.put("reOverTimeCount",reOverTimeCount + "");
        //已扫描数量
        map.put("isScanCount",isScanCount + "");
        //未扫描数量 已签收-已扫描
        map.put("notScanCount",(signCount - isScanCount) + "");
        //扫描率
        if(disCount != 0 && signCount != 0){
            scanRate = (isScanCount * 100 / signCount);
        }
        map.put("scanRate",scanRate + "%");
        return map;
    }

    private int getOverTimeCount(String startDateTo, String endDateTo) throws Exception{
        RecordSearch recordSearch = new RecordSearch();
        recordSearch.setStartDate(startDateTo);
        recordSearch.setEndDate(endDateTo);
        int count = 0;
        List<SubmitRecord> unSubmitRecords = recordMapper.selectUnCountGroupDept(recordSearch);
        if (!CollectionUtils.isEmpty(unSubmitRecords)) {
            unSubmitRecords = selectOvertimeRecord(unSubmitRecords);
            if(!CollectionUtils.isEmpty(unSubmitRecords)){
                count = unSubmitRecords.size();
            }
        }
        return count;
    }

    /**
     * 过滤未过期的
     * @param unSubmitRecords
     * @return
     */
    private List<SubmitRecord> selectOvertimeRecord(List<SubmitRecord> unSubmitRecords) throws Exception{
        //查询超期参数
        EmrOvertimeSet overtimeSet = overtimeSetService.selectByPrimaryKey(1);
        //获取未超期列表
        List<EmrHolidaySet> unHolidays = holidaySetMapper.selectNotHolidayList();
        Iterator<SubmitRecord> iterator = unSubmitRecords.iterator();
        //获取非死亡工作日
        String date = unHolidays.get(overtimeSet.getDays()).getDate();
        //获取非死亡工作日
        String deadDate = unHolidays.get(overtimeSet.getDeadDays()).getDate();
        //定义格式化日期
        SimpleDateFormat fmt = new SimpleDateFormat("yyyy-MM-dd");
        while (iterator.hasNext()) {
            SubmitRecord record = (SubmitRecord) iterator.next();
            //获取是否死亡
            String dischargeDisposition = record.getDischargeDisposition();
            String dischargeDateTime = record.getDischargeDateTime();
            long time = fmt.parse(dischargeDateTime).getTime();
            if(StringUtils.isNotBlank(dischargeDisposition) && "5".equals(dischargeDisposition)){
                //死亡
                if(time >= fmt.parse(deadDate).getTime()){
                    iterator.remove();
                }
            }else{
                //非死亡
                if(time >= fmt.parse(date).getTime()){
                    iterator.remove();
                }
            }
        }
        return unSubmitRecords;
    }

/*
    public Map<String,String> medicalRoomManageData(String startDateTo, String endDateTo) throws Exception{
        //查询总数据
        List<Archive_Master> list = archiveMasterMapper.recordDateOneTable(startDateTo, endDateTo, null);
        //组织返回数据
        Map<String, String> map = new HashMap<>();
        if (!CollectionUtils.isEmpty(list)) {
            //查询超期参数
            EmrOvertimeSet overtimeSet = overtimeSetService.selectByPrimaryKey(1);
            //获取未超期列表
            List<EmrHolidaySet> unHolidays = holidaySetMapper.selectNotHolidayList();
            //获取非死亡工作日
            String date = unHolidays.get(overtimeSet.getDays()).getDate();
            //获取死亡工作日
            String deadDate = unHolidays.get(overtimeSet.getDeadDays()).getDate();
            //获取预超期非死亡工作日
            String reDate = unHolidays.get(overtimeSet.getDays() - overtimeSet.getSurplusDays()).getDate();
            //获取预超期死亡工作日
            String reDeadDate = unHolidays.get(overtimeSet.getDeadDays() - overtimeSet.getSurplusDays()).getDate();
            //定义出院人数
            int disCount = 0;
            //定义待终审数量
            int headNurseCount = 0;
            //定义归档数量
            int recordCount = 0;
            //定义签收数量
            int signCount = 0;
            //超期病历
            int overTimeCount = 0;
            //预超期病历
            int reOverTimeCount = 0;
            //未扫描数量
            int notScanCount = 0;
            //定义格式化日期
            SimpleDateFormat fmt = new SimpleDateFormat("yyyy-MM-dd");
            //查询已纸质扫描的MasterId
            List<Archive_Detail> archiveDetails = archiveDetailMapper.selectIsScanByDate(startDateTo, endDateTo);
            for (Archive_Master obj : list) {
                //获取出院日期
                String dischargeDateTime = obj.getDischargeDateTime();
                //获取状态
                String archivestate = obj.getArchivestate();
                //获取签收时间
                String t1 = obj.getT1();
                //判断出院人数
                if (StringUtils.isNotBlank(dischargeDateTime) && !"1801".equals(dischargeDateTime.substring(0, 4))) {
                    disCount++;
                    //获取死亡状态
                    String dischargeDisposition = obj.getDischargeDisposition();
                    long time = fmt.parse(dischargeDateTime).getTime();
                    if(StringUtils.isNotBlank(dischargeDisposition) && "5".equals(dischargeDisposition)){
                        //死亡病历
                        if(time <= fmt.parse(deadDate).getTime()){
                            //超期
                            overTimeCount++;
                        }
                        if(time > fmt.parse(deadDate).getTime() && time <= fmt.parse(reDeadDate).getTime()){
                            //预超期
                            reOverTimeCount++;
                        }
                    }else{
                        //非死亡病历
                        if(time <= fmt.parse(date).getTime()){
                            //超期
                            overTimeCount++;
                        }
                        if(time > fmt.parse(date).getTime() && time <= fmt.parse(reDate).getTime()){
                            //预超期
                            reOverTimeCount++;
                        }
                    }
                }
                //判断待终审数量
                if ("64".equals(archivestate)) {
                    headNurseCount++;
                    //判断是否纸质扫描
                    boolean isScanFlag = false;
                    if(!CollectionUtils.isEmpty(archiveDetails)){
                        for(Archive_Detail detail : archiveDetails){
                            if(detail.getMasterid().equals(obj.getId())){
                                isScanFlag = true;
                                break;
                            }
                        }
                    }
                    if(!isScanFlag){
                        notScanCount++;
                    }
                }
                //判断归档数量
                if ("128".equals(archivestate)) {
                    recordCount++;
                }
                //判断签收数量
                if (StringUtils.isNotBlank(t1) && !"1801".equals(t1.substring(0, 4))) {
                    signCount++;
                }
            }
            //归档率 归档数量/出院人数
            map.put("recordRate", (recordCount * 100 / disCount) + "%");
            //终审数量
            map.put("recordCount", recordCount + "");
            //待终审数量
            map.put("headNurseCount", headNurseCount + "");
            //出科数量 待终审数量+终审数量
            map.put("submitCount", (recordCount + headNurseCount) + "");
            //签收数量
            map.put("signCount", signCount + "");
            //待签收数量 总出院人数-已签收数量
            map.put("notSignCount", (disCount - signCount) + "");
            //签收率 已签收/总出院人数
            map.put("signRate", (signCount * 100 / disCount) + "%");
            //超期数量
            map.put("overTimeCount", overTimeCount + "");
            //预超期数量
            map.put("reOverTimeCount",reOverTimeCount + "");
            //已扫描数量
            map.put("isScanCount",archiveDetails.size() + "");
            //预超期数量
            map.put("notScanCount",notScanCount + "");
            //扫描率
            map.put("scanRate",(archiveDetails.size() * 100 / disCount) + "%");
        }
        return map;
    }
*/

    /**
     * HIS退回病案统计数据
     * @param offset
     * @param limit
     * @param startDateTo
     * @param endDateTo
     * @return
     */
    public OffsetLimitPage callBackStatisticsData(Integer offset, Integer limit, String startDateTo, String endDateTo) {
        PageHelper.offsetPage(offset, limit);
        List<RecordCommonVo> list = archiveMasterMapper.callBackStatisticsData(startDateTo, endDateTo);
        return new OffsetLimitPage((Page) list);
    }

    /**
     * HIS退回病案统计数据详情
     * @param offset
     * @param limit
     * @param time
     * @return
     */
    public OffsetLimitPage callBackStatisticsDetail(Integer offset, Integer limit, String time,String startDateTo,String endDateTo) {
        PageHelper.offsetPage(offset, limit);
        List<ArchiveOtherExtVo> list = archiveOtherExtMapper.callBackStatisticsDetail(time,startDateTo,endDateTo);
        return new OffsetLimitPage((Page) list);
    }
}
