package com.emr.shiro;

import com.alibaba.fastjson.JSONObject;
import com.emr.entity.Power_User;
import com.emr.util.HttpClientUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.SecurityUtils;
import org.apache.shiro.authc.*;
import org.apache.shiro.authz.AuthorizationInfo;
import org.apache.shiro.authz.SimpleAuthorizationInfo;
import org.apache.shiro.realm.AuthorizingRealm;
import org.apache.shiro.session.Session;
import org.apache.shiro.subject.PrincipalCollection;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.HashSet;
import java.util.Set;

/**
 * @author HJL
 * @create 2019/4/29
 */
public class MyRealm extends AuthorizingRealm {
    @Value("${powerUrl}")
    private String powerUrl;


    /**
     * 授权查询回调函数, 进行鉴权但缓存中无用户的授权信息时调用.
     */
    @Override
    protected AuthorizationInfo doGetAuthorizationInfo(PrincipalCollection principalCollection) {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        Power_User user = (Power_User)request.getSession().getAttribute("CURRENT_USER");
        //获取登录时输入的用户名
        if (user != null) {
            Set<String> menus = user.getMenus();
            // 用户权限列表
            Set<String> permsSet = new HashSet<String>();
            if(null != menus && !menus.isEmpty()){
                for(String menu : menus){
                    if(StringUtils.isNoneBlank(menu)){
                        permsSet.add(menu);
                    }
                }
            }
            SimpleAuthorizationInfo info = new SimpleAuthorizationInfo();
            info.setStringPermissions(permsSet);
            return info;
        }else{
            return null;
        }
    }

    /**
     * 认证回调函数,登录时调用.
     */
    @Override
    protected AuthenticationInfo doGetAuthenticationInfo(AuthenticationToken authenticationToken) throws AuthenticationException {
        UsernamePasswordToken token = (UsernamePasswordToken) authenticationToken;
        //通过表单接收的用户名
        String username = token.getUsername();
        String userpwd = String.valueOf(token.getPassword());
        if (username != null && !"".equals(username)) {
            String url = powerUrl+"?sysFlag=emr_medical_record&userName="+ username;
            JSONObject obj = HttpClientUtils.httpGet(url);
            if(obj!=null){
                if((obj.getString("code")).equals("100")){
                    //当验证都通过后，把用户信息放在 session 里
                    Session session = SecurityUtils.getSubject().getSession();
                    session.setAttribute("menuList", obj);
                     session.setAttribute("userSession", username);
                    return new SimpleAuthenticationInfo(username, userpwd, getName());
                }
            }
        }
        return null;
    }
}
 

